--[[
		Recap 3.54 - Hawksy (on Elune) (modified for WoW 2.01)
				   - original by Gello (versions 3.32 and earlier)

		An AddOn for WoW to track and summarize all damage done around the user.
]]

Recap_Version = 3.54

--[[ local tables ]]

-- colors for the gauges
local gaugecolor = { ["DmgIn"] = { r=.66,g=.25,b=.25 }, ["DmgOut"] = { r=.25,g=.66,b=.35 }, ["Heal"] = { r=.25,g=.5,b=.85 },
					 ["Time"] = { r=.75, g=.75, b=.75 }, ["Kills"] = { r=.75, g=.75, b=.75 }, ["MaxHit"] = { r=.75, g=.75, b=.75 },
					 ["DPS"] = { r=.25, g=.66, b=.35 }, ["Over"] = { r=.25, g=.5, b=.85 } }

-- keys for (det)ails and (inc)oming data sets
local detkey =	{ ["HitsDmg"]="d", ["Hits"]="n", ["HitsMax"]="x", ["CritsDmg"]="D", ["Crits"]="N",
				  ["CritsMax"]="X", ["CritsEvents"]="e", ["Missed"]="m", ["TicksDmg"]="t", ["TicksMax"]="T" }
-- Hawksy: added NonMeleeTicks
local inckey = { ["MeleeDamage"]="a", ["MeleeMax"]="b", ["MeleeHits"]="c", ["MeleeCrits"]="d", ["MeleeMissed"]="e",
				 ["MeleeDodged"]="f", ["MeleeParried"]="g", ["MeleeBlocked"]="h", ["NonMeleeDamage"]="i",
				 ["NonMeleeMax"]="j", ["NonMeleeHits"]="k", ["NonMeleeCrits"]="l", ["NonMeleeMissed"]="m",
				 ["NonMeleeTicks"]="n" }

-- .Opt.x and its buttons
local recapbuttons = { ["Minimized"] = "RecapMinimizeButton", ["Pinned"] = "RecapPinButton",
					   ["Paused"] = "RecapPauseButton", ["SelfView"] = "RecapSelfViewButton" }

-- Hawksy: events with zero damage or healing will be ignored if they have only the following info keys
local zeroEventKeys = { "source", "victim", "type", "amount", "skill", "element" }

-- Hawksy: when dumping events to WoWChatLog, emit keys in a canonical order
local canonicalKeys = { "type", "source", "victim", "skill", "amount", "element", "isCrit", "isDOT", "amountResist", "amountAbsorb", "missType", "isItem", "isGlancing", "isSplit" }


--[[ Saved Variables ]]

recap = {} -- options and current data
recap_set = {} -- fight data sets

--[[ Callback functions ]]

function Recap_OnLoad()

	this:RegisterEvent("VARIABLES_LOADED")
	this:RegisterEvent("PLAYER_ENTERING_WORLD")
	this:RegisterEvent("UNIT_NAME_UPDATE")

	-- function overrides: recap.NormalExit is true on a successful /quit or /logout
	Recap_OldQuit = Quit
	Recap_OldLogout = Logout
	Recap_OldCancelLogout = CancelLogout
	Quit = function() recap.NormalExit = true; Recap_OldQuit(); end
	Logout = function() recap.NormalExit = true; Recap_OldLogout(); end
	CancelLogout = function() recap.NormalExit = nil; Recap_OldCancelLogout(); end

end

function Recap_OnEvent()

	if event=="VARIABLES_LOADED" then

		SlashCmdList["RecapCOMMAND"] = Recap_SlashHandler
		SLASH_RecapCOMMAND1 = "/Recap"

	elseif event=="PLAYER_ENTERING_WORLD" or event=="UNIT_NAME_UPDATE" then
		Recap_Initialize()

	elseif event=="PLAYER_REGEN_DISABLED" then
		if recap.Opt.LimitFights.value then
			Recap_StartFight()
		end

	elseif event=="PLAYER_REGEN_ENABLED" then
		Recap_EndFight()

	elseif event=="PARTY_MEMBERS_CHANGED" or event=="RAID_ROSTER_UPDATE" then
		Recap_MakeFriends()

	elseif event=="CHAT_MSG_CHANNEL_NOTICE" then
		if arg4 and string.find(arg4,"recaplog"..string.lower((UnitName("player")))) and arg1=="YOU_JOINED" then
			RecapWriteLogButton:SetText(RECAP_CHANNEL_JOINED_WRITING)
			if LoggingChat() then
				-- already on
			else
				-- toggle on
				LoggingChat(true)
			end
			RecapFrame:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE")
			RecapFrame:RegisterEvent("CHAT_MSG_CHANNEL")
			Recap_WriteLog()
		end

	elseif event=="CHAT_MSG_CHANNEL" then
		if arg4 and string.find(string.lower(arg4),"recaplog") and arg2==(UnitName("player")) and arg1=="EOF" then
			if not LoggingChat() then
				-- already off
			else
				-- toggle off
				LoggingChat(true)
			end
			RecapWriteLogButton:SetText(RECAP_SAVE_TO_WOWCHATLOG_TXT)
			LeaveChannelByName("recaplog"..(UnitName("player")))
			RecapFrame:UnregisterEvent("CHAT_MSG_CHANNEL")
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Local.LogWritten)
		end
	end
end

function Recap_Idle_OnUpdate(arg1)

	-- timer to end idle fights if option checked
	if recap_temp.IdleTimer~=-1 and recap_temp.Loaded and recap.Opt.IdleFight.value then
		recap_temp.IdleTimer = recap_temp.IdleTimer + arg1
		if recap_temp.IdleTimer > recap.Opt.IdleFightTimer.value then
			recap_temp.IdleTimer = -1
			Recap_EndFight()
		end
	end

	-- if fighting and a plugin loaded or we're minimized, update min dps every half second
	if recap_temp.Loaded and recap_temp.UpdateTimer~=-1 and recap.Opt.State.value=="Active" and
	  ((IB_Recap_Update or TitanPanelRecap_Update) or recap.Opt.Minimized.value) then
		recap_temp.UpdateTimer = recap_temp.UpdateTimer + arg1
		if recap_temp.UpdateTimer>.5 then
			recap_temp.UpdateTimer = 0
			Recap_UpdateMinimizedDPS()
		end
	end
end

function Recap_OnUpdate(arg1)

	-- update fade timer if option checked
	if recap_temp.Loaded and recap.Opt.AutoFade.value and not recap.Opt.Minimized.value and MouseIsOver(RecapFrame) and recap_temp.FadeTimer<recap.Opt.AutoFadeTimer.value then
		recap_temp.FadeTimer = 0
	elseif recap_temp.Loaded and recap_temp.FadeTimer~=-1 and recap.Opt.AutoFade.value and not recap.Opt.Minimized.value then
		recap_temp.FadeTimer = recap_temp.FadeTimer + arg1
		if recap_temp.FadeTimer > recap.Opt.AutoFadeTimer.value then
			RecapFrame:SetAlpha( max(1-min(2*(recap_temp.FadeTimer-recap.Opt.AutoFadeTimer.value),1),.1))
			if recap_temp.FadeTimer > (recap.Opt.AutoFadeTimer.value+.5) then
				RecapFrame_Hide()
			end
		end
	end

	if recap_temp.Loaded and recap.Opt.AutoMinimize.value and recap_temp.Selected==0 then
		if not recap.Opt.Minimized.value and not MouseIsOver(RecapFrame) and not IsShiftKeyDown() then
			Recap_Minimize()
		elseif recap.Opt.Minimized.value and MouseIsOver(RecapFrame) and not IsShiftKeyDown() then
			Recap_Maximize()
		end
	end

end

function Recap_SlashHandler(arg1)

	local i

	Recap_Initialize()

	if arg1 and arg1=="reset" then
		RecapFrame:ClearAllPoints()
		RecapFrame:SetPoint("CENTER","UIParent","CENTER")
		RecapOptFrame:ClearAllPoints()
		RecapOptFrame:SetPoint("CENTER","UIParent","CENTER")
		Recap_SetView()

	elseif arg1=="debug" then
		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug is ON. To turn it off: /recap debug off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug is OFF. To turn it on: /recap debug on")
		end
	elseif arg1=="debug on" then
		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was on, and is still ON.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was off, and is now ON.")
		end
		recap.debug = true
	elseif arg1=="debug off" then
		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.debug was off, and is still OFF.")
		end
		recap.debug = false

	-- Hawksy: temporary debug setting, to automatically screenshot
	elseif arg1=="snap" then
		if recap.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap.snap is ON. To turn it off: /recap snap off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.snap is OFF. To turn it on: /recap snap on")
		end
	elseif arg1=="snap on" then
		if recap.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap.snap was on, and is still ON.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.snap was off, and is now ON.")
		end
		recap_temp.snap = true
	elseif arg1=="snap off" then
		if recap.snap then
			DEFAULT_CHAT_FRAME:AddMessage("recap.snap was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.snap was off, and is still OFF.")
		end
		recap_temp.snap = false

	-- Hawksy: another debug setting, to dump ParserLib events to WoWChatLog.txt
	elseif arg1=="dump" then
		if recap.dump then
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump is ON. To turn it off: /recap dump off")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump is OFF. To turn it on: /recap dump on")
		end
	elseif arg1=="dump on" then
		local i, chatname, freeslot
		local logchat = "recaplog"..string.lower((UnitName("player")))
		local headerformat = recap_temp.Local.LogHeader
		freeslot = false
		for i=1,10 do
			_,chatname = GetChannelName(i)
			if not chatname then
				freeslot = true
			elseif string.lower(chatname)==logchat then
				freeslot = true
			end
		end
		if freeslot then
			-- not using event "CHAT_MSG_CHANNEL_NOTICE" at the moment, probably need to do so
			JoinChannelByName(logchat)
			-- Hawksy: may need to be a significant delay here to let things work
			-- for the moment, an "off" "on" sequence until you get the "failed to join" message, then one more "on" -- bizzare
			-- ah, LoggingChat toggles, rather than sets, the state
			recap.dumpchatnum = nil
			for i=1,10 do
				j,chatname = GetChannelName(i)
				if chatname and string.lower(chatname)==logchat then
					recap.dumpchatnum=j
					break
				end
			end
			if recap.dumpchatnum then
				-- we're in a channel, turn on logging and write a header
				if LoggingChat() then
					-- already on
				else
					-- toggle on
					LoggingChat(true)
				end
				SendChatMessage(string.format(headerformat,(UnitName("player")),recap_temp.Local.LastAll[recap.Opt.View.value],date(),GetRealZoneText()),"CHANNEL",nil,recap.dumpchatnum)
				if recap.dump then
					DEFAULT_CHAT_FRAME:AddMessage("recap.dump was on, and is still ON.")
				else
					DEFAULT_CHAT_FRAME:AddMessage("recap.dump was off, and is now ON.")
				end
				recap.dump = true
				recap.dumplines = 0
			else
				-- not in a channel after all
				DEFAULT_CHAT_FRAME:AddMessage("recap.dump failed to join channel, try again.")
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Local.NoFreeChannels)
		end
	elseif arg1=="dump off" then
		if recap.dumpchatnum then
			if not LoggingChat() then
				-- already off
			else
				-- toggle off
				LoggingChat(true)
			end
			LeaveChannelByName("recaplog"..(UnitName("player")))
			recap.dumpchatnum = nil
		end
		if recap.dump then
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump was on, and is now OFF.")
		else
			DEFAULT_CHAT_FRAME:AddMessage("recap.dump was off, and is still OFF.")
		end
		recap.dump = false

	elseif arg1 and string.find(arg1,"opt") then
		if RecapOptFrame:IsShown() then
			RecapOptFrame:Hide()
		else
			RecapOptFrame:Show()
		end

	elseif RecapFrame:IsShown() then
		RecapFrame_Hide()
		RecapOptFrame:Hide()

	else
		RecapFrame_Show()
		Recap_CheckWindowBounds()
		if recap.Opt.State.value == "Stopped" then
			Recap_SetState("Idle")
			recap.Opt.Paused.value = false
			Recap_SetButtons()
		end
	end
end

--[[ Window appearance functions ]]

function Recap_SetState(arg1)
	if arg1 then
		recap.Opt.State.value = arg1
	end
	if recap.Opt.State.value=="Stopped" then
		RecapStatusTexture:SetVertexColor(1,0,0)
		Recap_Unregister_CombatEvents()
	elseif recap.Opt.State.value=="Active" then
		RecapStatusTexture:SetVertexColor(0,1,0)
		Recap_Register_CombatEvents()
	else
		RecapStatusTexture:SetVertexColor(.5,.5,.5)
		Recap_Register_CombatEvents()
	end
	Recap_UpdatePlugins()
end

function RecapFrame_Hide()

	if not recap_temp.Loaded then
		Recap_Initialize()
	end
	recap_temp.FadeTimer = -1
	RecapFrame:SetAlpha(1)
	RecapFrame:Hide()
	recap.Opt.Visible.value = false
end

function RecapFrame_Show()

	if not recap_temp.Loaded then
		Recap_Initialize()
	end
	RecapFrame:Show()
	recap.Opt.Visible.value = true
end

function RecapOptFrame_Hide()
--[[
	RecapSetEditBox:SetText("")
	RecapSetEditBox:ClearFocus()
	if RecapOptClipFrame:IsVisible() then
		RecapClipEditBox:ClearFocus()
		RecapOptClipFrame:Hide()
		RecapOptSubFrame4:Show()
	end
]]
	RecapOptFrame:Hide()
end

function RecapFrame_Toggle()
	if RecapFrame:IsShown() then
		RecapFrame_Hide()
		RecapOptFrame:Hide()
	else
		RecapFrame_Show()
	end
end

function Recap_Shutdown()

	recap.Opt.Paused.value = true
	Recap_EndFight()
	Recap_SetState("Stopped")
	Recap_SetButtons()
	RecapOptFrame:Hide()
	RecapFrame_Hide()
	RecapCombatEvents:UnregisterAllEvents()
	recap_temp.parserlib:UnregisterAllEvents("Recap")
	Recap_SetEventRange(true)
end

function Recap_ToggleView()

	if recap.Opt.View.value=="All" then
		Recap_SetView("Last")
	else
		Recap_SetView("All")
	end
end

-- changes button textures depending on their state
function Recap_SetButtons()

	local i

	for i in pairs(recapbuttons) do
		if recap.Opt[i].value then
			getglobal(recapbuttons[i]):SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
			getglobal(recapbuttons[i]):SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
		else
			getglobal(recapbuttons[i]):SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
			getglobal(recapbuttons[i]):SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
		end
	end

	if recap.Opt.View.value=="Last" then
		RecapShowAllButton:SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
		RecapShowAllButton:SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
	else
		RecapShowAllButton:SetNormalTexture("Interface\\AddOns\\Recap\\RecapButtons-Down")
		RecapShowAllButton:SetPushedTexture("Interface\\AddOns\\Recap\\RecapButtons-Up")
	end
end

-- this resizes the window width to the columns defined in options
function Recap_SetColumns()

	local i, j, cx, item, subitem

	-- Hawksy: name width was 120 changed to 144 to allow for longer names, especially
	--   under Spells and Abilities on the "Personal Details" panel.
	-- Matching changes made in Recap.xml for parent_EName, parent_Name, RecapHeader_Name, and
	--   RecapHeader_EName.
	cx = recap_temp.ScrollAndEdgeWidth + recap_temp.NameWidth

	if recap.Opt.SelfView.value then
		-- personal view
		for i in pairs(recap.Opt) do
			if recap_temp.DefaultOpt[i].ewidth then
				if recap.Opt[i].value then
					getglobal("RecapHeader_"..i):SetWidth(recap_temp.DefaultOpt[i].ewidth)
					getglobal("RecapHeader_"..i):Show()
					for j=1,15 do
						getglobal("RecapSelf"..j.."_"..i):SetWidth(recap_temp.DefaultOpt[i].ewidth)
						getglobal("RecapSelf"..j.."_"..i):Show()
					end
					cx = cx + recap_temp.DefaultOpt[i].ewidth
				else
					getglobal("RecapHeader_"..i):SetWidth(1)
					getglobal("RecapHeader_"..i):Hide()
					for j=1,15 do
						getglobal("RecapSelf"..j.."_"..i):SetWidth(1)
						getglobal("RecapSelf"..j.."_"..i):Hide()
					end
					cx = cx + 1
				end
			elseif recap_temp.DefaultOpt[i].width then
				getglobal("RecapHeader_"..i):Hide()
			end
		end

		RecapTotals:Hide()
		RecapHeader_Name:Hide()
		RecapHeader_EName:Show()

		for i=1,15 do
			getglobal("RecapList"..i):Hide()
			getglobal("RecapSelf"..i):Show()
			getglobal("RecapSelf"..i):SetWidth(cx - recap_temp.ScrollAndEdgeWidth)
		end

	else
		-- general view
		for i in pairs(recap.Opt) do
			if recap_temp.DefaultOpt[i].width then
				if recap.Opt[i].value then
					-- column will appear
					item = getglobal("RecapHeader_"..i)
					item:SetWidth(recap_temp.DefaultOpt[i].width)
					item:Show()
					item = getglobal("RecapTotals_"..i)
					item:SetWidth(recap_temp.DefaultOpt[i].width)
					item:Show()
					-- Hawksy: Faction and Level always show and hide together
					if i == "Faction" then
						-- Hawksy: split the width (hardcoded numbers)
						item:SetWidth(14)
						subitem = getglobal("RecapTotals_Level")
						subitem:SetWidth(18)
						subitem:Show()
					end
					for j=1,15 do
						item = getglobal("RecapList"..j.."_"..i)
						item:SetWidth(recap_temp.DefaultOpt[i].width)
						item:Show()
						-- Hawksy: Faction and Level always show and hide together
						if i == "Faction" then
							-- Hawksy: split the width (hardcoded numbers)
							item:SetWidth(14)
							subitem = getglobal("RecapList"..j.."_Level")
							subitem:SetWidth(18)
							subitem:Show()
						end
					end
					cx = cx + recap_temp.DefaultOpt[i].width
				else
					-- column will not appear
					item = getglobal("RecapHeader_"..i)
					item:SetWidth(1)
					item:Hide()
					-- Hawksy: Class column fix (see below)
					if i == "Class" then
						item:Show()
					end
					item = getglobal("RecapTotals_"..i)
					item:SetWidth(1)
					item:Hide()
					-- Hawksy: Class column fix (see below)
					if i == "Class" then
						item:Show()
					end
					-- Hawksy: Faction and Level always show and hide together
					if i == "Faction" then
						subitem = getglobal("RecapTotals_Level")
						subitem:SetWidth(1)
						subitem:Hide()
						cx = cx + 1 -- two columns here, not just one
					end
					for j=1,15 do
						item = getglobal("RecapList"..j.."_"..i)
						item:SetWidth(1)
						item:Hide()
						-- Hawksy: Class column should not appear, but to fix
						--   a display bug (i.e. if Class is not shown and other
						--   columns are hidden or revealed the RecapList is not
						--   re-positioned, a bug I haven't yet been able to
						--   explain) we show it as a single column of transparent
						--   black.  See also RecapScrollBar_Update and Recap_ConstructList.
						if i == "Class" then
							item:Show()
						end
						-- Hawksy: Faction and Level always show and hide together
						if i == "Faction" then
							subitem = getglobal("RecapList"..j.."_Level")
							subitem:SetWidth(1)
							subitem:Hide()
						end
					end
					cx = cx + 1
				end
			elseif recap_temp.DefaultOpt[i].ewidth then
				getglobal("RecapHeader_"..i):Hide()
			end
		end

		for i=1,15 do
			getglobal("RecapSelf"..i):Hide()
			getglobal("RecapList"..i):Show()
			getglobal("RecapList"..i):SetWidth(cx - recap_temp.ScrollAndEdgeWidth)
		end

		RecapTotals:Show()
		RecapHeader_Name:Show()
		RecapHeader_EName:Hide()

	end

	-- Hawksy: the name slot on the Total line is narrower
	RecapTotals_Name:SetWidth(recap_temp.NameWidth-44)
	RecapTotals:SetWidth(cx-44)

	if recap.Opt.GrowLeftwards.value and RecapFrame then
		i = RecapFrame:GetRight()
		j = RecapFrame:GetTop()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",i-cx,j) -- *** i not defined sometimes :( **
		end
	end

	RecapTopBar:SetWidth(cx-8)
	RecapBottomBar:SetWidth(cx-16)
	RecapFrame:SetWidth(cx)
	RecapScrollBar:SetWidth(cx-16)

	recap_temp.GaugeWidth = cx - recap_temp.ScrollAndEdgeWidth - 3 -
							( (not recap.Opt.SelfView.value) and
							   ((((recap.Opt.Faction.value and recap.Opt.Faction.width) or 1) +
								 ((recap.Opt.Class.value and recap.Opt.Class.width) or 1) +
								 ((recap.Opt.Ranks.value and recap.Opt.Ranks.width) or 1)) or 0) or 0)

	Recap_CheckWindowBounds()
	Recap_SetView()
	RecapScrollBar_Update()

end

-- changes view mode
function Recap_SetView(arg1)

	local text

	if arg1 then
		recap.Opt.View.value = arg1
	end

	Recap_SetButtons()
	Recap_ConstructList()

	cx = RecapFrame:GetWidth()

	-- change titlebar
	if cx and cx>300 and recap.Opt.SelfView.value then
		text = recap_temp.Player.."'s Details"
	elseif cx and cx>230 and recap.Opt.SelfView.value then
		text = recap_temp.Player
	elseif cx and cx<230 or recap.Opt.Minimized.value then
		text = ""
		if cx>180 and not recap.Opt.SelfView.value then
			text = recap.Opt.View.value
		end
	elseif cx and cx<260 then
		text = recap_temp.Local.LastAll[recap.Opt.View.value]
	else
		text = "Recap of "..recap_temp.Local.LastAll[recap.Opt.View.value]
	end

	RecapTitle:SetText(text)
	RecapHeader_Name:SetText((recap_temp.ListSize-1).." Combatants")
	Recap_ChangeBack()

end

function Recap_SetColors()

	local i

	if recap.Opt.UseColor.value then
		recap_temp.ColorDmgIn = recap_temp.ColorRed
		recap_temp.ColorDmgOut = recap_temp.ColorGreen
		recap_temp.ColorHeal = recap_temp.ColorBlue
		recap_temp.ColorHits = recap_temp.ColorCyan
		recap_temp.ColorTicks = recap_temp.ColorYellow
		recap_temp.ColorCrits = recap_temp.ColorLime
		recap_temp.ColorMiss = recap_temp.ColorPink
	else
		recap_temp.ColorDmgIn = recap_temp.ColorWhite
		recap_temp.ColorDmgOut = recap_temp.ColorNone
		recap_temp.ColorHeal = recap_temp.ColorWhite
		recap_temp.ColorHits = recap_temp.ColorWhite
		recap_temp.ColorTicks = recap_temp.ColorWhite
		recap_temp.ColorCrits = recap_temp.ColorWhite
		recap_temp.ColorMiss = recap_temp.ColorWhite
	end

	RecapTotals_DmgIn:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapTotals_DmgOut:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_HPS:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapTotals_DPSIn:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapTotals_DPS:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_DPSvsAll:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	RecapTotals_Heal:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapTotals_Over:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
	RecapMinDPSIn_Text:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
	RecapMinDPSOut_Text:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
	for i=1,15 do
		getglobal("RecapList"..i.."_HealP"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
		getglobal("RecapList"..i.."_Over"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
		getglobal("RecapList"..i.."_DmgInP"):SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
		getglobal("RecapList"..i.."_DmgOutP"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		getglobal("RecapSelf"..i.."_EHits"):SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		getglobal("RecapSelf"..i.."_EHitsAvg"):SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		getglobal("RecapSelf"..i.."_EHitsMax"):SetTextColor(recap_temp.ColorHits.r,recap_temp.ColorHits.g,recap_temp.ColorHits.b)
		getglobal("RecapSelf"..i.."_ETicks"):SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		getglobal("RecapSelf"..i.."_ETicksAvg"):SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		getglobal("RecapSelf"..i.."_ETicksMax"):SetTextColor(recap_temp.ColorTicks.r,recap_temp.ColorTicks.g,recap_temp.ColorTicks.b)
		getglobal("RecapSelf"..i.."_ECrits"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_ECritsAvg"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_ECritsMax"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_ECritsP"):SetTextColor(recap_temp.ColorCrits.r,recap_temp.ColorCrits.g,recap_temp.ColorCrits.b)
		getglobal("RecapSelf"..i.."_EMiss"):SetTextColor(recap_temp.ColorMiss.r,recap_temp.ColorMiss.g,recap_temp.ColorMiss.b)
		getglobal("RecapSelf"..i.."_EMissP"):SetTextColor(recap_temp.ColorMiss.r,recap_temp.ColorMiss.g,recap_temp.ColorMiss.b)
	end
	Recap_ChangeBack()
end

function Recap_ChangeBack()

	if recap.Opt.UseColor.value and (recap.Opt.MinBack.value or not recap.Opt.Minimized.value) then

		if recap.Opt.SelfView.value and not recap.Opt.Minimized.value then
			RecapTitleBack:SetVertexColor(0,0,.8)
		elseif recap.Opt.View.value=="Last" then
			RecapTitleBack:SetVertexColor(0,.66,0)
		else
			RecapTitleBack:SetVertexColor(.66,0,0)
		end
		RecapTitleBack:Show()
	else
		RecapTitleBack:Hide()
	end
end

-- Hawksy: new function for setting the event range
function Recap_SetEventRange(forceDefault)

	if recap.Opt.MaxEventRange.value and not forceDefault then
		-- maximum
		SetCVar("CombatDeathLogRange", 200)
		SetCVar("CombatLogRangeParty", 200)
		SetCVar("CombatLogRangePartyPet", 200)
		SetCVar("CombatLogRangeFriendlyPlayers", 200)
		SetCVar("CombatLogRangeFriendlyPlayersPets", 200)
		SetCVar("CombatLogRangeHostilePlayers", 200)
		SetCVar("CombatLogRangeHostilePlayersPets", 200)
		SetCVar("CombatLogRangeCreature", 200)
	else
		-- Blizzard default
		SetCVar("CombatDeathLogRange", 60)
		SetCVar("CombatLogRangeParty", 50)
		SetCVar("CombatLogRangePartyPet", 50)
		SetCVar("CombatLogRangeFriendlyPlayers", 50)
		SetCVar("CombatLogRangeFriendlyPlayersPets", 50)
		SetCVar("CombatLogRangeHostilePlayers", 50)
		SetCVar("CombatLogRangeHostilePlayersPets", 50)
		SetCVar("CombatLogRangeCreature", 30)
	end
end


--[[ Initialization ]]

-- Can be called often, will immediately dump out if it ran through successfully
function Recap_Initialize()

	local i, x
	local init_time

	if not recap_temp.Loaded then

		if (UnitName("player")) and (UnitName("player"))~=UNKNOWNOBJECT and RecapFrame and RecapOptFrame then

			init_time = GetTime()

			recap_temp.Player = (UnitName("player"))
			recap_temp.Server = GetCVar("realmName")

			-- recap index for this player
			recap_temp.p = recap_temp.Player..recap_temp.Server -- use for stuff that can be made global
			recap_temp.s = recap_temp.Player..recap_temp.Server -- use for stuff that should never be global

			if recap and recap.UseOneSettings and recap.User then
				recap_temp.p = recap.User
			end

			-- Hawksy: force the dump flags to nil, otherwise we could wake up spamming a channel other than our proper log channel
			recap.dump = nil
			recap.dumpchatnum = nil

			Recap_InitializeData()

			if recap.Opt.RecentData.value then
				Recap_CreateRecent()
			else
				recap_temp.Recent = {}
			end

			if not recap[recap_temp.p].WindowTop and RecapFrame then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("CENTER","UIParent","CENTER")
				RecapOptFrame:ClearAllPoints()
				RecapOptFrame:SetPoint("CENTER", "UIParent", "CENTER")
			elseif recap[recap_temp.p].WindowTop and RecapFrame then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",recap[recap_temp.p].WindowLeft,recap[recap_temp.p].WindowTop)
				RecapFrame:SetWidth(recap[recap_temp.p].WindowWidth)
				RecapFrame:SetHeight(recap[recap_temp.p].WindowHeight)
			end

			for i in pairs(recap.Combatant) do
				recap.Combatant[i].WasInLast = false
				recap.Combatant[i].WasInCurrent = false
			end
			recap[recap_temp.p].LastDuration = 0

			Recap_SetColors()

			Recap_SetEventRange(false)

			-- if we loaded out of stopped state, force into idle state
			if recap.Opt.State.value ~= "Stopped" then
				recap.Opt.State.value = "Idle"
			end
			Recap_SetState()

			-- Hawksy: was 82 now 144
			RecapTotals_Name:SetWidth(recap_temp.NameWidth)

			Recap_InitDataSets()

			if recap.Opt.LightData.value then
				recap.Opt.SelfView.value = false
				RecapSelfViewButton:Hide()
			end

			if recap.Opt.Minimized.value then
				Recap_SetButtons()
				Recap_ConstructList()
				Recap_Minimize()
			else
				Recap_Maximize()
			end

			-- we're in the world (in theory), register active events
			this:RegisterEvent("PLAYER_REGEN_DISABLED")
			this:RegisterEvent("PLAYER_REGEN_ENABLED")
			this:RegisterEvent("PARTY_MEMBERS_CHANGED")
			this:RegisterEvent("RAID_ROSTER_UPDATE")

			Recap_Register_CombatEvents()

			recap_temp.Loaded = true

			Recap_InitializeOptions()
			Recap_MoveMinimize()

			-- initialize panel
			recap_temp.Selected = 0
			if recap.Opt.LightData.value then
				recap.Opt.PanelView.value = 3
				RecapPanelTab1:SetAlpha(.5)
				RecapPanelTab2:SetAlpha(.5)
			end
			RecapPanel_SwitchPanels(recap.Opt.PanelView.value)
			if recap.Opt.PanelAnchor.value then
				RecapPanel:ClearAllPoints()
				RecapPanel:SetPoint(recap.Opt.PanelAnchor.Panel,"RecapFrame",recap.Opt.PanelAnchor.Main,Recap_PanelOffset("x"),Recap_PanelOffset("y"))
			end

			-- Hawksy: removed the Alpha reduction for Faction icons

			if recap.Opt.Visible.value then
				RecapFrame_Show()
			else
				RecapFrame_Hide()
			end

			if recap.Opt.Paused.value then
				RecapFrame_Hide()
			end

			if recap.debug then
				DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Time to initialize: "..string.format("%.4f",GetTime()-init_time).." seconds.")
			end

			-- warn if user has a lot of data (over 500 combatants)
			x = 0
			for i in pairs(recap.Combatant) do
				x = x + 1
			end
			if x>500 and recap.Opt.WarnData.value then
				StaticPopupDialogs["RECAP_LOTS_OF_DATA_WARNING"] = {
					text = TEXT(string.format(recap_temp.Local.LotsOfDataWarning,x)),
					button1 = TEXT(OKAY),
					timeout = 0,
					showAlert = 1,
					whileDead = 1
				}
				StaticPopup_Show("RECAP_LOTS_OF_DATA_WARNING")
			end

			table.insert(UISpecialFrames,"RecapOptFrame")
			table.insert(UISpecialFrames,"RecapPanel")

		end

	end

end

-- arg1==1 to hide buttons, 16 to show
function Recap_ShowButtons(arg1)

	RecapCloseButton:SetWidth(arg1)
	RecapPinButton:SetWidth(arg1)
	RecapPauseButton:SetWidth(arg1)
	RecapOptionsButton:SetWidth(arg1)
	RecapShowAllButton:SetWidth(arg1)

	if arg1==16 then
		RecapCloseButton:Show()
		RecapPinButton:Show()
		RecapPauseButton:Show()
		RecapOptionsButton:Show()
		RecapShowAllButton:Show()
	else
		RecapCloseButton:Hide()
		RecapPinButton:Hide()
		RecapPauseButton:Hide()
		RecapOptionsButton:Hide()
		RecapShowAllButton:Hide()
	end
end

function Recap_Minimize()

	local i, j
	local cx = 20 -- 8(edge)+4(space right of status)+8

	recap.Opt.Minimized.value = true

	for i in pairs(recap.Opt) do
		if recap_temp.DefaultOpt[i].minwidth then
			if recap.Opt[i].value then
				item = getglobal("Recap"..i)
				item:SetWidth(recap_temp.DefaultOpt[i].minwidth)
				item:Show()
				cx = cx + recap_temp.DefaultOpt[i].minwidth
			else
				item = getglobal("Recap"..i)
				item:SetWidth(1)
				item:Hide()
				cx = cx + 1
			end
		elseif recap_temp.DefaultOpt[i].width or recap_temp.DefaultOpt[i].ewidth then
			getglobal("RecapHeader_"..i):Hide()
		end
	end

	if recap.Opt.MinButtons.value then
		cx = cx + 108
		Recap_ShowButtons(16)
	else
		Recap_ShowButtons(1)
		cx = cx + 23 -- 5 + 18
	end

	RecapTitle:SetText(recap.Opt.View.value)

	RecapTopBar:Hide()
	RecapBottomBar:Hide()
	RecapResetButton:Hide()
	for i=1,15 do
		getglobal("RecapList"..i):Hide()
		getglobal("RecapSelf"..i):Hide()
	end
	RecapTotals:Hide()
	RecapHeader_Name:Hide()
	RecapHeader_EName:Hide()
	RecapScrollBar:Hide()

	if recap.Opt.AutoMinimize.value then
		cx = cx-15
	end

	if recap.Opt.GrowLeftwards.value and RecapFrame then
		i = RecapFrame:GetRight()
		j = RecapFrame:GetTop()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",i-cx,j)
		end
	end
	RecapFrame:SetWidth(cx)
	Recap_SetHeight(28)

	if recap.Opt.MinBack.value then
		RecapFrame:SetBackdropColor(1,1,1,1)
		RecapFrame:SetBackdropBorderColor(.5,.5,.5,1)
	else
		RecapFrame:SetBackdropColor(0,0,0,0)
		RecapFrame:SetBackdropBorderColor(0,0,0,0)
	end

	RecapSelfViewButton:Hide()
	RecapPanel_Hide(1)
	Recap_MoveMinimize()
	Recap_ChangeBack()

	RecapMinView:GetLeft()

end

function Recap_Maximize()

	local i

	recap.Opt.Minimized.value = false

	Recap_SetColumns()

	for i in pairs(recap.Opt) do
		if recap_temp.DefaultOpt[i].minwidth then
			item = getglobal("Recap"..i)
			item:SetWidth(recap_temp.DefaultOpt[i].minwidth)
			item:Hide()
		end
	end
	RecapMinStatus:Show()
	RecapMinView:Show()
	Recap_ShowButtons(16)

	RecapTopBar:Show()
	RecapBottomBar:Show()
	RecapResetButton:Show()

	RecapScrollBar:Show()

	if recap.Opt.SelfView.value then
		RecapTotals:Hide()
		RecapHeader_Name:Hide()
		RecapHeader_EName:Show()
	else
		RecapTotals:Show()
		RecapHeader_Name:Show()
		RecapHeader_EName:Hide()
	end

	RecapFrame:SetBackdropColor(1,1,1,1)
	RecapFrame:SetBackdropBorderColor(.5,.5,.5,1)
	if not recap.Opt.LightData.value then
		RecapSelfViewButton:Show()
	end
	Recap_MoveMinimize()

end

function Recap_SetHeight(newcy)

	local i,j

	if recap.Opt.GrowUpwards.value and RecapFrame then
		i = RecapFrame:GetBottom()
		j = RecapFrame:GetLeft()
		if i and j then
			RecapFrame:ClearAllPoints()
			RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",j,i+newcy)
		end
	end

	RecapFrame:SetHeight(newcy)
	Recap_CheckWindowBounds()
end

function RecapScrollBar_Update()

	local i, j, index, item
	local listsize = recap.Opt.SelfView.value and recap_temp.SelfListSize or recap_temp.ListSize
	local thisCombatant

	if not recap.Opt.Minimized.value then

		i = 72 + ( math.max( math.min(listsize-1,recap.Opt.MaxRows.value),1 )*14 )
		Recap_SetHeight(i)
		RecapScrollBar:SetHeight(i-63)
		FauxScrollFrame_Update(RecapScrollBar,listsize-1,recap.Opt.MaxRows.value,10)

		for i=1,recap.Opt.MaxRows.value do
			index = i + FauxScrollFrame_GetOffset(RecapScrollBar)

			if not recap.Opt.SelfView.value then
				-- general view

				if index < listsize then
					-- Hawksy: guard code
					thisCombatant = recap_temp.List[index].Name
					if thisCombatant and recap.Combatant[thisCombatant] then

						getglobal("RecapList"..i.."_Ranks"):SetText(index..".")

						Recap_SetFactionIcon(i,recap.Combatant[thisCombatant].Faction)
						getglobal("RecapList"..i.."_Level"):SetText(" ") -- default
						if recap.Opt.Faction.value then
							if recap.Combatant[thisCombatant].Level==-1 then
								getglobal("RecapList"..i.."_Level"):SetText("??")
							elseif recap.Combatant[thisCombatant].Level and tonumber(recap.Combatant[thisCombatant].Level)>0 then
								getglobal("RecapList"..i.."_Level"):SetText(recap.Combatant[thisCombatant].Level)
							end
						end

						-- Hawksy: Class column should not appear, but to fix
						--   a display bug (i.e. if Class is not shown and other
						--   columns are hidden or revealed the RecapList is not
						--   re-positioned) we show it as a single column of black.
						--   See more code in Recap_SetColumns and Recap_ConstructList,
						--   and a texture addition to RecapHeader_Faction.
						if recap.Opt["Class"].value then
							-- Class will appear
							Recap_SetClassIcon(i,recap.Combatant[thisCombatant].Class)
						else
							-- Class icon needs to be set to black
							Recap_SetClassIcon(i,nil)
						end

						item = getglobal("RecapList"..i.."_Name")
						item:SetText(thisCombatant)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = getglobal("RecapList"..i.."_Gauge")
						if recap.Opt.ShowGauges.value and recap_temp.GaugeMax>0 and recap.Combatant[thisCombatant].Friend and recap_temp.GaugeBy then
							item:SetWidth(1+recap_temp.GaugeWidth*recap_temp.List[index][recap_temp.GaugeBy]/recap_temp.GaugeMax)
							item:Show()
						else
							item:Hide()
						end

						getglobal("RecapList"..i.."_Kills"):SetText(recap_temp.List[index].Kills)

						getglobal("RecapList"..i.."_Time"):SetText(Recap_FormatTime(recap_temp.List[index].Time))

						item = getglobal("RecapList"..i.."_Heal")
						item:SetText(recap_temp.List[index].Heal)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						getglobal("RecapList"..i.."_HealP"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].HealP.."%" or "")
						getglobal("RecapList"..i.."_Over"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].Over.."%" or "")

						getglobal("RecapList"..i.."_MaxHit"):SetText(recap_temp.List[index].MaxHit)

						item = getglobal("RecapList"..i.."_DmgIn")
						item:SetText(recap_temp.List[index].DmgIn)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						getglobal("RecapList"..i.."_DmgInP"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].DmgInP.."%" or "")

						item = getglobal("RecapList"..i.."_DmgOut")
						item:SetText(recap_temp.List[index].DmgOut)
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						getglobal("RecapList"..i.."_DmgOutP"):SetText(recap.Combatant[thisCombatant].Friend and recap_temp.List[index].DmgOutP.."%" or "")

						item = getglobal("RecapList"..i.."_HPS")
						item:SetText(string.format(recap_temp.List[index].HPS<1000 and "%.1f" or "%d",recap_temp.List[index].HPS))
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end
						item = getglobal("RecapList"..i.."_DPSIn")
						item:SetText(string.format(recap_temp.List[index].DPSIn<1000 and "%.1f" or "%d",recap_temp.List[index].DPSIn))
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end
						item = getglobal("RecapList"..i.."_DPS")
						item:SetText(string.format(recap_temp.List[index].DPS<1000 and "%.1f" or "%d",recap_temp.List[index].DPS))
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = getglobal("RecapList"..i.."_DPSvsAll")
						item:SetText(string.format(recap_temp.List[index].DPSvsAll<1000 and "%.1f" or "%d",recap_temp.List[index].DPSvsAll))
						if recap.Combatant[thisCombatant].Friend then
							item:SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						else
							item:SetTextColor(recap_temp.ColorWhite.r,recap_temp.ColorWhite.g,recap_temp.ColorWhite.b)
						end

						item = getglobal("RecapList"..i)
						item:Show()
						if recap_temp.Selected == index then
							item:LockHighlight()
						else
							item:UnlockHighlight()
						end
					end
				else
					getglobal("RecapList"..i):Hide()
					getglobal("RecapList"..i):UnlockHighlight()
				end

				for i=recap.Opt.MaxRows.value+1,15 do
					getglobal("RecapList"..i):Hide()
				end

			elseif recap.Opt.SelfView.value then
				-- personal view

				if index < listsize then

					getglobal("RecapSelf"..i.."_EName"):SetText(string.sub(recap_temp.SelfList[index].EName,2))
					getglobal("RecapSelf"..i.."_ETotal"):SetText(recap_temp.SelfList[index].ETotal)
					getglobal("RecapSelf"..i.."_EHits"):SetText(recap_temp.SelfList[index].EHits)
					getglobal("RecapSelf"..i.."_ETicks"):SetText(recap_temp.SelfList[index].ETicks)
					getglobal("RecapSelf"..i.."_ECrits"):SetText(recap_temp.SelfList[index].ECrits)
					getglobal("RecapSelf"..i.."_EHitsAvg"):SetText(recap_temp.SelfList[index].EHitsAvg)
					getglobal("RecapSelf"..i.."_ETicksAvg"):SetText(recap_temp.SelfList[index].ETicksAvg)
					getglobal("RecapSelf"..i.."_ECritsAvg"):SetText(recap_temp.SelfList[index].ECritsAvg)
					getglobal("RecapSelf"..i.."_EHitsMax"):SetText(recap_temp.SelfList[index].EHitsMax)
					getglobal("RecapSelf"..i.."_ETicksMax"):SetText(recap_temp.SelfList[index].ETicksMax)
					getglobal("RecapSelf"..i.."_ECritsMax"):SetText(recap_temp.SelfList[index].ECritsMax)
					getglobal("RecapSelf"..i.."_ECritsP"):SetText(recap_temp.SelfList[index].ECritsP)
					getglobal("RecapSelf"..i.."_EMiss"):SetText(recap_temp.SelfList[index].EMiss)
					getglobal("RecapSelf"..i.."_EMissP"):SetText(recap_temp.SelfList[index].EMissP)
					getglobal("RecapSelf"..i.."_EMaxAll"):SetText(recap_temp.SelfList[index].EMaxAll)
					getglobal("RecapSelf"..i.."_ETotalP"):SetText(recap_temp.SelfList[index].ETotalP)
					if string.sub(recap_temp.SelfList[index].EName,1,1)<"3" then
						getglobal("RecapSelf"..i.."_EName"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						getglobal("RecapSelf"..i.."_ETotal"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
						getglobal("RecapSelf"..i.."_ETotalP"):SetTextColor(recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
					else
						getglobal("RecapSelf"..i.."_EName"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						getglobal("RecapSelf"..i.."_ETotal"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
						getglobal("RecapSelf"..i.."_ETotalP"):SetTextColor(recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
					end
					getglobal("RecapSelf"..i):Show()
				else
					getglobal("RecapSelf"..i):Hide()
				end

				for i=recap.Opt.MaxRows.value+1,15 do
					getglobal("RecapSelf"..i):Hide()
				end

			end

		end

	end
end

--[[ Fight functions ]]

function Recap_CreateCombatantAndUpdateTimers(thisCombatant, event_time, timeType)

	-- guard code
	if not thisCombatant then
		return
	end

	-- if thisCombatant not a .Combatant, add an entry for them
	if not recap.Combatant[thisCombatant] then
		Recap_CreateBlankCombatant(thisCombatant)
	end

	-- if they haven't fought this session, add a .Last entry for them
	if not recap_temp.Last[thisCombatant] then
		recap_temp.Last[thisCombatant] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, Heal = 0 }
	end

	-- Hawksy: If we get here for any reason we might as well start the overall fight timer
	--   (Recap 3.32 would only do so for a source doing damage, or for a spellcast start)
	--   (Is healing after a fight, thus starting a new fight, any different from someone ganking a toad?)
	if recap_temp.FightStart==0 then
		recap_temp.FightStart = event_time
		recap_temp.FightEnd = recap_temp.FightStart
	else
		recap_temp.FightEnd = event_time
	end

	-- Hawksy: three independent timers per combatant for damage dealt, damage received, and healing done
	--   (at the moment no timer for healing received)
	if timeType == "Out" then
		if recap_temp.Last[thisCombatant].StartOut==0 then
			recap_temp.Last[thisCombatant].StartOut = event_time
			recap_temp.Last[thisCombatant].EndOut = recap_temp.Last[thisCombatant].StartOut
		else
			recap_temp.Last[thisCombatant].EndOut = event_time
		end
	elseif timeType == "In" then
		if recap_temp.Last[thisCombatant].StartIn==0 then
			recap_temp.Last[thisCombatant].StartIn = event_time
			recap_temp.Last[thisCombatant].EndIn = recap_temp.Last[thisCombatant].StartIn
		else
			recap_temp.Last[thisCombatant].EndIn = event_time
		end
	elseif timeType == "Heal" then
		if recap_temp.Last[thisCombatant].StartHeal==0 then
			recap_temp.Last[thisCombatant].StartHeal = event_time
			recap_temp.Last[thisCombatant].EndHeal = recap_temp.Last[thisCombatant].StartHeal
		else
			recap_temp.Last[thisCombatant].EndHeal = event_time
		end
	end

	-- anyone who does damage, takes damage, or heals will be flagged as having
	--   been in the fight, but simply getting healed still doesn't count
	recap.Combatant[thisCombatant].WasInCurrent = true
end

function Recap_StartFight()

	local i

	if recap.Opt.State.value=="Idle" then
		if recap.Opt.AutoHide.value and not recap.Opt.Minimized.value then
			RecapFrame_Hide()
		end
		for i in pairs(recap.Combatant) do
			recap.Combatant[i].WasInCurrent = false
		end
		if not recap.Opt.LimitFights.value and recap.Opt.IdleFight.value then
			recap_temp.IdleTimer = 0
		end
		RecapIdleFrame:Show() -- start idle OnUpdates
		Recap_SetState("Active")

		-- Hawksy: check friends at start of fight to get HP for more accurate
		--   calculation of overhealing.  See also Recap_GetUnitInfo.
		Recap_MakeFriends()
	end
end

-- returns the current DPS of combatant named arg1 if it exists. UpdateDuration/DamageOut reused to save allocations
function Recap_GetCurrentDPS(arg1)

	if arg1 and recap_temp.Last[arg1] and recap_temp.Last[arg1].DmgOut>0 then
		recap_temp.UpdateDuration = recap_temp.Last[arg1].EndOut - recap_temp.Last[arg1].StartOut
		if recap.Opt.View.value=="All" then
			recap_temp.UpdateDuration = recap_temp.UpdateDuration + recap.Combatant[arg1].TotalTime
		end
		recap_temp.UpdateDamageOut = recap_temp.Last[arg1].DmgOut
		if recap.Opt.View.value=="All" then
			recap_temp.UpdateDamageOut = recap_temp.UpdateDamageOut + recap.Combatant[arg1].TotalDmgOut
		end

		if recap_temp.UpdateDuration > recap_temp.MinTime then
			return (recap_temp.UpdateDamageOut / recap_temp.UpdateDuration)
		end
	end

	return 0
end

function Recap_UpdateMinimizedDPS()

	local i

	if recap.Opt.State.value=="Active" then

		if recap_temp.Last[recap_temp.Player] and recap_temp.Last[recap_temp.Player].DmgOut>0 then
			i = Recap_GetCurrentDPS(recap_temp.Player)+Recap_GetCurrentDPS((UnitName("pet")))
			RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
		end

		-- calculate running total DPS In and Out
		recap_temp.UpdateDuration = 0
		if recap_temp.FightEnd>0 and recap_temp.FightStart>0 then
			recap_temp.UpdateDuration = recap_temp.FightEnd - recap_temp.FightStart
		end
		if recap.Opt.View.value=="All" then
			recap_temp.UpdateDuration = recap_temp.UpdateDuration + recap[recap_temp.p].TotalDuration
		end

		if recap_temp.UpdateDuration>recap_temp.MinTime then

			recap_temp.UpdateDmgIn = 0
			recap_temp.UpdateDmgOut = 0
			for i in pairs(recap_temp.Last) do
				if recap.Combatant[i].Friend then
					recap_temp.UpdateDmgIn = recap_temp.UpdateDmgIn + recap_temp.Last[i].DmgIn
					recap_temp.UpdateDmgOut = recap_temp.UpdateDmgOut + recap_temp.Last[i].DmgOut
				end
			end

			if recap.Opt.View.value=="All" then
				for i in pairs(recap.Combatant) do
					if recap.Combatant[i].Friend then
						recap_temp.UpdateDmgIn = recap_temp.UpdateDmgIn + recap.Combatant[i].TotalDmgIn
						recap_temp.UpdateDmgOut = recap_temp.UpdateDmgOut + recap.Combatant[i].TotalDmgOut
					end
				end
			end

			RecapMinDPSIn_Text:SetText(string.format("%.1f",recap_temp.UpdateDmgIn/recap_temp.UpdateDuration))
			RecapMinDPSOut_Text:SetText(string.format("%.1f",recap_temp.UpdateDmgOut/recap_temp.UpdateDuration))

		end

		Recap_UpdatePlugins()
	end
end

function Recap_AddPetToOwner(pet,owner)

	local i
	local peteffect

	-- Hawksy: guard code
	if not recap.Combatant[pet] then
		return
	end
	if not recap.Combatant[owner] then
		return
	end
	if not recap_temp.Last[pet] then
		return
	end
	if not recap_temp.Last[owner] then
		return
	end

	-- Hawksy: now three independent times, for damage, incoming, and healing
	if (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartOut > 0)) and
	   (recap.Combatant[owner].WasInCurrent and (recap_temp.Last[owner].StartOut > 0)) then
		-- both pet and owner were in the fight and did damage
		recap_temp.Last[owner].StartOut = math.min(recap_temp.Last[pet].StartOut,recap_temp.Last[owner].StartOut)
		recap_temp.Last[owner].EndOut = math.max(recap_temp.Last[pet].EndOut,recap_temp.Last[owner].EndOut)
	elseif (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartOut > 0)) and
		   (not recap.Combatant[owner].WasInCurrent or (recap_temp.Last[owner].StartOut == 0)) then
		-- the pet was in the fight and did damage, the owner did no damage
		recap_temp.Last[owner].StartOut = recap_temp.Last[pet].StartOut
		recap_temp.Last[owner].EndOut = recap_temp.Last[pet].EndOut
	end
	if (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartIn > 0)) and
	   (recap.Combatant[owner].WasInCurrent and (recap_temp.Last[owner].StartIn > 0)) then
		-- both pet and owner were in the fight and took damage
		recap_temp.Last[owner].StartIn = math.min(recap_temp.Last[pet].StartIn,recap_temp.Last[owner].StartIn)
		recap_temp.Last[owner].EndIn = math.max(recap_temp.Last[pet].EndIn,recap_temp.Last[owner].EndIn)
	elseif (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartIn > 0)) and
		   (not recap.Combatant[owner].WasInCurrent or (recap_temp.Last[owner].StartIn == 0)) then
		-- the pet was in the fight and took damage, the owner took no damage
		recap_temp.Last[owner].StartIn = recap_temp.Last[pet].StartIn
		recap_temp.Last[owner].EndIn = recap_temp.Last[pet].EndIn
	end
	if (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartHeal > 0)) and
	   (recap.Combatant[owner].WasInCurrent and (recap_temp.Last[owner].StartHeal > 0)) then
		-- both pet and owner were in the fight and did healing
		recap_temp.Last[owner].StartHeal = math.min(recap_temp.Last[pet].StartHeal,recap_temp.Last[owner].StartHeal)
		recap_temp.Last[owner].EndHeal = math.max(recap_temp.Last[pet].EndHeal,recap_temp.Last[owner].EndHeal)
	elseif (recap.Combatant[pet].WasInCurrent and (recap_temp.Last[pet].StartHeal > 0)) and
		   (not recap.Combatant[owner].WasInCurrent or (recap_temp.Last[owner].StartHeal == 0)) then
		-- the pet was in the fight and did healing, the owner did no healing
		recap_temp.Last[owner].StartHeal = recap_temp.Last[pet].StartHeal
		recap_temp.Last[owner].EndHeal = recap_temp.Last[pet].EndHeal
	end

	recap_temp.Last[owner].DmgIn = recap_temp.Last[owner].DmgIn + recap_temp.Last[pet].DmgIn
	recap_temp.Last[owner].DmgOut = recap_temp.Last[owner].DmgOut + recap_temp.Last[pet].DmgOut
	recap_temp.Last[owner].MaxHit = math.max(recap_temp.Last[owner].MaxHit,recap_temp.Last[pet].MaxHit)
	-- Hawksy: add in heals and kills
	recap_temp.Last[owner].Kills = recap_temp.Last[owner].Kills + recap_temp.Last[pet].Kills
	recap_temp.Last[owner].Heal = recap_temp.Last[owner].Heal + recap_temp.Last[pet].Heal

	recap_temp.Last[pet].StartOut = 0
	recap_temp.Last[pet].EndOut = 0
	recap_temp.Last[pet].StartIn = 0
	recap_temp.Last[pet].EndIn = 0
	recap_temp.Last[pet].StartHeal = 0
	recap_temp.Last[pet].EndHeal = 0
	recap_temp.Last[pet].DmgIn = 0
	recap_temp.Last[pet].DmgOut = 0
	recap_temp.Last[pet].MaxHit = 0
	recap_temp.Last[pet].Kills = 0
	recap_temp.Last[pet].Heal = 0

	Recap_AccumulateCombatant(owner)

	recap.Combatant[pet].WasInCurrent = false
	recap.Combatant[owner].WasInCurrent = false
	recap.Combatant[pet].WasInLast = false
	recap.Combatant[owner].WasInLast = true

	if not recap.Opt.LightData.value then
		for i in pairs(recap.Combatant[pet].Detail) do
			if string.sub(i,1,1)=="1" then
				peteffect = "2"..pet.."'s "..string.sub(i,2)
			else
				peteffect = "4"..pet.."'s "..string.sub(i,2)
			end
			recap.Combatant[owner].Detail[peteffect] = {
				Hits = recap.Combatant[pet].Detail[i].Hits, HitsDmg = recap.Combatant[pet].Detail[i].HitsDmg,
				HitsMax = recap.Combatant[pet].Detail[i].HitsMax, Crits = recap.Combatant[pet].Detail[i].Crits,
				CritsEvents = recap.Combatant[pet].Detail[i].CritsEvents, CritsDmg = recap.Combatant[pet].Detail[i].CritsDmg,
				CritsMax = recap.Combatant[pet].Detail[i].CritsMax, Missed = recap.Combatant[pet].Detail[i].Missed,
				TicksDmg = recap.Combatant[pet].Detail[i].TicksDmg, TicksMax = recap.Combatant[pet].Detail[i].TicksMax }
			if owner==recap_temp.Player then
				recap.Self[recap_temp.s][peteffect] = {
					Hits = recap.Combatant[pet].Detail[i].Hits, HitsDmg = recap.Combatant[pet].Detail[i].HitsDmg,
					HitsMax = recap.Combatant[pet].Detail[i].HitsMax, Crits = recap.Combatant[pet].Detail[i].Crits,
					CritsEvents = recap.Combatant[pet].Detail[i].CritsEvents, CritsDmg = recap.Combatant[pet].Detail[i].CritsDmg,
					CritsMax = recap.Combatant[pet].Detail[i].CritsMax, Missed = recap.Combatant[pet].Detail[i].Missed,
					TicksDmg = recap.Combatant[pet].Detail[i].TicksDmg, TicksMax = recap.Combatant[pet].Detail[i].TicksMax }
			end
		end
	end

end

function Recap_AccumulateCombatant(i)

	recap.Combatant[i].LastDmgIn = recap_temp.Last[i].DmgIn
	recap.Combatant[i].LastDmgOut = recap_temp.Last[i].DmgOut
	recap.Combatant[i].LastTime = recap_temp.Last[i].EndOut - recap_temp.Last[i].StartOut
	recap.Combatant[i].LastTimeIn = recap_temp.Last[i].EndIn - recap_temp.Last[i].StartIn
	recap.Combatant[i].LastTimeHeal = recap_temp.Last[i].EndHeal - recap_temp.Last[i].StartHeal
	recap.Combatant[i].LastMaxHit = recap_temp.Last[i].MaxHit
	recap.Combatant[i].LastKills = recap_temp.Last[i].Kills
	recap.Combatant[i].LastHeal = recap_temp.Last[i].Heal

	-- Hawksy: If there is non-zero damage or healing, force the matching time to be
	--   (somewhat arbitrarily) at least as large as the global cooldown ("DeemedTime").
	--   This gives some duration to single-shot fights (which are accumulated), keeping
	--   the anomalous DPS down a bit, without displaying them or their DPS.
	if (recap.Combatant[i].LastDmgOut > 0) and (recap.Combatant[i].LastTime < recap_temp.DeemedTime) then
		recap.Combatant[i].LastTime = recap_temp.DeemedTime
	end
	if (recap.Combatant[i].LastDmgIn > 0) and (recap.Combatant[i].LastTimeIn < recap_temp.DeemedTime) then
		recap.Combatant[i].LastTimeIn = recap_temp.DeemedTime
	end
	if (recap.Combatant[i].LastHeal > 0) and (recap.Combatant[i].LastTimeHeal < recap_temp.DeemedTime) then
		recap.Combatant[i].LastTimeHeal = recap_temp.DeemedTime
	end

	recap_temp.Last[i].StartOut = 0
	recap_temp.Last[i].EndOut = 0
	recap_temp.Last[i].StartIn = 0
	recap_temp.Last[i].EndIn = 0
	recap_temp.Last[i].StartHeal = 0
	recap_temp.Last[i].EndHeal = 0
	recap_temp.Last[i].DmgIn = 0
	recap_temp.Last[i].DmgOut = 0
	recap_temp.Last[i].MaxHit = 0
	recap_temp.Last[i].Kills = 0
	recap_temp.Last[i].Heal = 0

	recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn + recap.Combatant[i].LastDmgIn
	recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut + recap.Combatant[i].LastDmgOut
	recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime + recap.Combatant[i].LastTime
	recap.Combatant[i].TotalTimeIn = recap.Combatant[i].TotalTimeIn + recap.Combatant[i].LastTimeIn
	recap.Combatant[i].TotalTimeHeal = recap.Combatant[i].TotalTimeHeal + recap.Combatant[i].LastTimeHeal
	recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills + recap.Combatant[i].LastKills
	recap.Combatant[i].TotalHeal = recap.Combatant[i].TotalHeal + recap.Combatant[i].LastHeal

	-- Hawksy: three flavours of "per second" calculation
	if recap.Combatant[i].LastTime > recap_temp.MinTime then
		recap.Combatant[i].LastDPS = recap.Combatant[i].LastDmgOut/recap.Combatant[i].LastTime
	else
		recap.Combatant[i].LastDPS = 0
	end
	if recap.Combatant[i].TotalTime > recap_temp.MinTime then
		recap.Combatant[i].TotalDPS = recap.Combatant[i].TotalDmgOut/recap.Combatant[i].TotalTime
	else
		recap.Combatant[i].TotalDPS = 0
	end
	if recap.Combatant[i].LastTimeIn > recap_temp.MinTime then
		recap.Combatant[i].LastDPSIn = recap.Combatant[i].LastDmgIn/recap.Combatant[i].LastTimeIn
	else
		recap.Combatant[i].LastDPSIn = 0
	end
	if recap.Combatant[i].TotalTimeIn > recap_temp.MinTime then
		recap.Combatant[i].TotalDPSIn = recap.Combatant[i].TotalDmgIn/recap.Combatant[i].TotalTimeIn
	else
		recap.Combatant[i].TotalDPSIn = 0
	end
	if recap.Combatant[i].LastTimeHeal > recap_temp.MinTime then
		recap.Combatant[i].LastHPS = recap.Combatant[i].LastHeal/recap.Combatant[i].LastTimeHeal
	else
		recap.Combatant[i].LastHPS = 0
	end
	if recap.Combatant[i].TotalTimeHeal > recap_temp.MinTime then
		recap.Combatant[i].TotalHPS = recap.Combatant[i].TotalHeal/recap.Combatant[i].TotalTimeHeal
	else
		recap.Combatant[i].TotalHPS = 0
	end

	recap.Combatant[i].TotalMaxHit = math.max(recap.Combatant[i].LastMaxHit,recap.Combatant[i].TotalMaxHit)

	recap.Combatant[i].WasInCurrent = false
	recap.Combatant[i].WasInLast = true
end


-- Hawksy: Special case for combined DoT/HoT spells where the healing event comes via ParserLib as if it were a non-HoT
-- This code is designed to correct existing errors after the fact
function Recap_CorrectDualDoTHoT()

	local thisCombatant, thisTypeEffect, thisType, healEffect, damageEffect

	-- first for the Combatant table
	for thisCombatant in pairs(recap.Combatant) do
		for thisTypeEffect in pairs(recap.Combatant[thisCombatant].Detail) do
			-- checking every effect for every combatant
			thisType = tonumber(string.sub(thisTypeEffect,1,1))
			if (thisType >= 3) then
				-- only check from healing effects
				healEffect = thisTypeEffect
				-- compute the matching damage effect, and see if it exists
				damageEffect = tostring(thisType-2)..string.sub(healEffect,2)
				if recap.Combatant[thisCombatant].Detail[damageEffect] then
					-- there is a matching damage effect, so this is a dual damage/healing effect
					if recap.Combatant[thisCombatant].Detail[damageEffect].TicksDmg and recap.Combatant[thisCombatant].Detail[healEffect].HitsDmg then
						-- the damage half of the effect is a DoT and the healing half is non-HoT healing
						-- convert the healing half of the effect into a HoT (note that there is no change to Hits)
						recap.Combatant[thisCombatant].Detail[healEffect].CritsEvents = nil
						recap.Combatant[thisCombatant].Detail[healEffect].TicksDmg = (recap.Combatant[thisCombatant].Detail[healEffect].TicksDmg or 0) + (recap.Combatant[thisCombatant].Detail[healEffect].HitsDmg or 0)
						recap.Combatant[thisCombatant].Detail[healEffect].HitsDmg = nil
						recap.Combatant[thisCombatant].Detail[healEffect].TicksMax = math.max((recap.Combatant[thisCombatant].Detail[healEffect].TicksMax or 0),(recap.Combatant[thisCombatant].Detail[healEffect].HitsMax or 0))
						recap.Combatant[thisCombatant].Detail[healEffect].HitsMax = nil
					end
				end
			end
		end
	end

	-- second for the Self table
	for thisTypeEffect in pairs(recap.Self[recap_temp.s]) do
		-- checking every effect for every combatant
		thisType = tonumber(string.sub(thisTypeEffect,1,1))
		if (thisType >= 3) then
			-- only check from healing effects
			healEffect = thisTypeEffect
			-- compute the matching damage effect, and see if it exists
			damageEffect = tostring(thisType-2)..string.sub(healEffect,2)
			if recap.Self[recap_temp.s][damageEffect] then
				-- there is a matching damage effect, so this is a dual damage/healing effect
				if recap.Self[recap_temp.s][damageEffect].TicksDmg and recap.Self[recap_temp.s][healEffect].HitsDmg then
					-- the damage half of the effect is a DoT and the healing half is non-HoT healing
					-- convert the healing half of the effect into a HoT (note that there is no change to Hits)
					recap.Self[recap_temp.s][healEffect].CritsEvents = nil
					recap.Self[recap_temp.s][healEffect].TicksDmg = (recap.Self[recap_temp.s][healEffect].TicksDmg or 0) + (recap.Self[recap_temp.s][healEffect].HitsDmg or 0)
					recap.Self[recap_temp.s][healEffect].HitsDmg = nil
					recap.Self[recap_temp.s][healEffect].TicksMax = math.max((recap.Self[recap_temp.s][healEffect].TicksMax or 0),(recap.Self[recap_temp.s][healEffect].HitsMax or 0))
					recap.Self[recap_temp.s][healEffect].HitsMax = nil
				end
			end
		end
	end
end


function Recap_EndFight()

	local i

	recap_temp.IdleTimer = -1

	RecapIdleFrame:Hide()

	if recap.Opt.State.value=="Active" then

		recap.Opt.State.value = "Stopped" -- suspend logging to calculate

		local debug_time = GetTime()

		Recap_MakeFriends()

		-- Hawksy: set to false as part of a fix to ensure that pet owners always get a chance
		--   to show up in the Last panel when Merge is on
		for i in pairs(recap.Combatant) do
			recap.Combatant[i].WasInLast = false
		end

		if recap_temp.FightEnd>recap_temp.FightStart then
			recap[recap_temp.p].LastDuration = recap_temp.FightEnd-recap_temp.FightStart
		else
			recap[recap_temp.p].LastDuration = 0
		end
		recap[recap_temp.p].TotalDuration = recap[recap_temp.p].TotalDuration + recap[recap_temp.p].LastDuration
		recap_temp.FightStart = 0
		recap_temp.FightEnd = 0

		for i in pairs(recap_temp.Last) do

			if recap.Combatant[i].WasInCurrent then
				-- Hawksy: one of these three will set WasInLast to true if appropriate
				if recap.Combatant[i].OwnedBy and recap.Opt.MergePets.value then
					Recap_AddPetToOwner(i,recap.Combatant[i].OwnedBy)
				elseif recap.Combatant[i].OwnsPet and recap.Opt.MergePets.value then
					Recap_AddPetToOwner(recap.Combatant[i].OwnsPet,i)
				else
					Recap_AccumulateCombatant(i)
				end
			else
				-- Hawksy: don't set WasInLast here, it could exclude an owner from the Last panel
				--   of the owner is processed later than the pet when Merge is true
			end

		end

		-- Hawksy: if Recent Data Mode is active, add an 'end of fight' event
		--   (signalled by nil value for source)
		if recap.Opt.RecentData.value then
			Recap_AddRecentEvent(debug_time, nil, nil, nil, nil, nil, nil)
		end

		-- Hawksy: special case correction for some dual damage/healing tickers
		Recap_CorrectDualDoTHoT()

		if recap.Opt.AutoPost.value then
			recap.Opt.SortBy.value = Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat)
			recap.Opt.SortDir.value = false
		end

		Recap_ConstructList()
		Recap_SetState("Idle")

		if not recap.Opt.Minimized.value and recap.Opt.AutoHide.value then
			RecapFrame_Show()
			if recap.Opt.AutoFade.value then
				recap_temp.FadeTimer = 0
			end
		end

		if recap.Opt.AutoPost.value then
			Recap_PostFight()
		end

		if recap.Opt.AutoLeader.value then
			Recap_PostLeader()
		end

		if recap.debug then
			DEFAULT_CHAT_FRAME:AddMessage("(recap.debug) Fight processed in "..string.format("%.4f",GetTime()-debug_time).." seconds for "..(recap_temp.ListSize-1).." combatants.")
		end

	end
end


-- Hawksy: dump ParserLib events to WoWChatLog.txt
function DumpMessage(message)
	if recap.dump then
		-- include a line count so that we can detect if any lines get dropped
		recap.dumplines = recap.dumplines + 1
		SendChatMessage(tostring(recap.dumplines)..";"..message,"CHANNEL",nil,recap.dumpchatnum)
	end
end

function CreateMessage(event, info)
	local index, value, printable, knownIndex, j

	printable = event
	-- first, 'known' keys in a fixed order
	for j=1,#canonicalKeys do
		index = canonicalKeys[j]
		if info[index] then
			value = tostring(info[index])
			value = string.gsub(value,"\ ","\_") -- convert blank to underscore
			printable = printable..";"..index.."="..value
		end
	end
	-- second, 'unknown' keys if any, in arbitrary order
	for index in pairs(info) do
		knownIndex = false
		for j=1,#canonicalKeys do
			if index == canonicalKeys[j] then
				-- the index is one of the 'known' ones
				knownIndex = true
				break
			end
		end
		if not knownIndex then
			-- this is an 'unknown' key
			value = tostring(info[index])
			value = string.gsub(value,"\ ","\_") -- convert blank to underscore
			printable = printable..";"..index.."="..value
		end
	end
	return printable
end


-- adds CHAT_MSG_SPELL/COMBAT parsings (from ParserLib) to .Last running totals
ParserLib_SELF = 103
ParserLib_MELEE = 112
ParserLib_DAMAGESHIELD = 113
function Recap_DamageEvent(event, info)

	local source, dest, damage, effect
	local sourceOkay, destOkay
	local event_time = GetTime()

	if recap.Opt.State.value~="Stopped" then

		if recap.dump then
			local printable
			printable = CreateMessage(event, info)
			DumpMessage("DAMAGE;"..printable)
		end

		source = info.source
		if tonumber(source) == ParserLib_SELF then
			source = recap_temp.Player
		end
		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end
		-- Hawksy: there are still occasional appearances of the personal pronoun despite ParserLib
		--   (apparently only in the destination slot, but we check both)
		if recap_temp.Local.SelfPronouns[string.lower(source)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_DamageEvent duplicate arg1 ("..tostring(arg1)..") event "..tostring(event).." source "..tostring(source))
if recap_temp.snap then
	Screenshot()
end
			source = recap_temp.Player
		end
		if recap_temp.Local.SelfPronouns[string.lower(dest)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_DamageEvent duplicate arg1 ("..tostring(arg1)..") event "..tostring(event).." dest "..tostring(dest))
if recap_temp.snap then
	Screenshot()
end
			dest = recap_temp.Player
		end

		damage = 0
		damage = tonumber(info.amount)
		effect = info.skill
		if tonumber(effect) == ParserLib_MELEE then
			effect = "Melee"
		end
		if tonumber(effect) == ParserLib_DAMAGESHIELD then
			effect = "Damage Shield"
		end

		-- Hawksy: check for known duplicates
		if Recap_CheckForDuplicates(source, effect) then
			return
		end

		if (recap.Combatant[source] and recap.Combatant[source].Ignore) or (recap.Combatant[dest] and recap.Combatant[dest].Ignore) then
			return -- one or the other is an ignored combatant, pretend the hit didn't happen at all
		end

		-- Hawksy: if in Self Data Mode check whether the source and dest are the player (or players's pet)
		sourceOkay = Recap_CheckSelfMode(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy))
		destOkay = Recap_CheckSelfMode(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy))

		-- perhaps formally start the fight
		if not recap.Opt.LimitFights.value then
			if recap.Opt.IdleFight.value then
				recap_temp.IdleTimer = 0
			end
			if recap.Opt.State.value=="Idle" then
				Recap_StartFight()
			end
		end

		-- Hawksy: events are already almost entirely filtered down to damage or miss, so let's start the timers
		--   (we're currently not receiving events for casts that have hit but done no damage)
		if sourceOkay then
			Recap_CreateCombatantAndUpdateTimers(source, event_time, "Out")
		end
		if destOkay then
			Recap_CreateCombatantAndUpdateTimers(dest, event_time, "In")
		end

		-- remove friend flag from hostiles no longer charmed
		if sourceOkay and recap.Combatant[source].Friend and destOkay and recap.Combatant[dest].Friend then
			recap.Combatant[source].Friend = false
		end

		if damage and (damage > 0) then
			-- process totals
			if sourceOkay then
				recap_temp.Last[source].DmgOut = recap_temp.Last[source].DmgOut + damage
				if damage > recap_temp.Last[source].MaxHit then
					recap_temp.Last[source].MaxHit = damage
				end
			end
			if destOkay then
				recap_temp.Last[dest].DmgIn = recap_temp.Last[dest].DmgIn + damage
				if recap_temp.Last[dest].HP then
					recap_temp.Last[dest].HP = recap_temp.Last[dest].HP - damage
				end
			end

			if not recap.Opt.LightData.value then
				if destOkay then
					if (string.find(event,"SPELL")~=nil) or (info.element~=nil) then
						-- a spell, assume that this is non-melee
						-- or if it has an element (e.g. Frost), assume that this is non-melee
						recap.Combatant[dest].Incoming.NonMeleeDamage = (recap.Combatant[dest].Incoming.NonMeleeDamage or 0) + damage
						if (info.isDOT == true) then
							-- tick
							recap.Combatant[dest].Incoming.NonMeleeTicks = (recap.Combatant[dest].Incoming.NonMeleeTicks or 0) + 1
						else
							if (info.isCrit == true) then
								recap.Combatant[dest].Incoming.NonMeleeCrits = (recap.Combatant[dest].Incoming.NonMeleeCrits or 0) + 1
							else
								recap.Combatant[dest].Incoming.NonMeleeHits = (recap.Combatant[dest].Incoming.NonMeleeHits or 0) + 1
							end
						end
						recap.Combatant[dest].Incoming.NonMeleeMax = math.max(damage,recap.Combatant[dest].Incoming.NonMeleeMax or 0)
					else
						-- otherwise, assume that it is melee damage
						-- Note that for "Incoming" data, hit count and crit count are kept separate.
						-- On the other hand, for overall damage "CritsEvents" contains the sum of hit count and crit count.
						-- The two styles of counting are different, and require care.
						recap.Combatant[dest].Incoming.MeleeDamage = (recap.Combatant[dest].Incoming.MeleeDamage or 0) + damage
						if (info.isCrit == true) then
							recap.Combatant[dest].Incoming.MeleeCrits = (recap.Combatant[dest].Incoming.MeleeCrits or 0) + 1
						else
							recap.Combatant[dest].Incoming.MeleeHits = (recap.Combatant[dest].Incoming.MeleeHits or 0) + 1
						end
						recap.Combatant[dest].Incoming.MeleeMax = math.max(damage,recap.Combatant[dest].Incoming.MeleeMax or 0)
					end
				end

				if sourceOkay then
					-- determine whether it could have critted
					if string.find(event,"PERIODIC")~=nil then
						-- periodic event, assume not crit-capable, pass a 'nil' in the crit slot
						Recap_CreateEffect(source,effect,1,damage,nil,info.isDOT)
					else
						-- assume all others are crit capable, so pass on the value of isCrit
						if (info.isCrit == nil) then
							-- if no value for isCrit, call it false
							Recap_CreateEffect(source,effect,1,damage,false,info.isDOT)
						else
							Recap_CreateEffect(source,effect,1,damage,info.isCrit,info.isDOT)
						end
					end
				end
			end

			-- Hawksy: support for Recent Data Mode
			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(event_time, source, dest, 1, effect, damage, info.isCrit)
			end
		else
			-- no actual damage, probably a miss
			if not recap.Opt.LightData.value then
				if (info.type == "miss") then
					if destOkay then
						if (string.find(event,"SPELL")~=nil) or (info.element~=nil) then
							-- a spell, assume that this is non-melee
							-- or if it has an element (e.g. Frost), assume that this is non-melee
							-- non-melee misses are not distinguised by type (deflect, immune, evade, resist, reflect, absorb)
							recap.Combatant[dest].Incoming.NonMeleeMissed = (recap.Combatant[dest].Incoming.NonMeleeMissed or 0) + 1
						else
							-- melee miss
							if (info.missType == "miss") then
								-- ordinary melee miss
								recap.Combatant[dest].Incoming.MeleeMissed = (recap.Combatant[dest].Incoming.MeleeMissed or 0) + 1
							elseif (info.missType == "absorb") then
								-- Hawksy: melee absorb is treated as an ordinary melee miss
								recap.Combatant[dest].Incoming.MeleeMissed = (recap.Combatant[dest].Incoming.MeleeMissed or 0) + 1
							elseif (info.missType == "dodge") then
								-- melee dodge
								recap.Combatant[dest].Incoming.MeleeDodged = (recap.Combatant[dest].Incoming.MeleeDodged or 0) + 1
							elseif (info.missType == "parry") then
								-- melee parry
								recap.Combatant[dest].Incoming.MeleeParried = (recap.Combatant[dest].Incoming.MeleeParried or 0) + 1
							elseif (info.missType == "block") then
								-- melee block
								recap.Combatant[dest].Incoming.MeleeBlocked = (recap.Combatant[dest].Incoming.MeleeBlocked or 0) + 1
							else
								-- other misses treat as ordinary melee misses
								-- Hawksy: probably all misses are caught by one of the categories above
								recap.Combatant[dest].Incoming.MeleeMissed = (recap.Combatant[dest].Incoming.MeleeMissed or 0) + 1
							end
						end
					end

					if sourceOkay then
						-- register the miss as a non-crit-capable event so that the crit percentage calculations are accurate
						Recap_CreateEffect(source,effect,1,0,nil,nil)
					end

					-- Hawksy: support for Recent Data Mode
					if recap.Opt.RecentData.value then
						local missText = (info.missType or "miss")
						Recap_AddRecentEvent(event_time, source, dest, 1, effect, missText, nil)
					end
				else
					-- Hawksy: no damage and not a miss
					-- ignore zero damage events that have keys only from zeroEventKeys (source, victim, amount, type, skill, element)
					local i, j, allOkayIndexes
					allOkayIndexes = true
					for i in pairs(info) do
						allOkayIndexes = false
						for j=1,#zeroEventKeys do
							if i == zeroEventKeys[j] then
								-- the index is one of the okay ones
								allOkayIndexes = true
								break
							end
						end
						if not allOkayIndexes then
							-- we got through zeroEventKeys without a match for this index
							break
						end
					end
					if allOkayIndexes then
						-- probably an uninteresting event, do nothing, ignore the event
					else
						-- an unexpected event, post diagnostic information
local printable
printable = CreateMessage(event, info)
printable = "Recap_DamageEvent no damage and not a miss, ignored arg1 ("..tostring(arg1)..") "..printable
DumpMessage(printable)
printable = string.gsub(printable,"\;","\ ") -- convert semicolon to blank for ordinary output
DEFAULT_CHAT_FRAME:AddMessage(printable)
if recap_temp.snap then
	Screenshot()
end
					end
				end
			end
		end
	end
end


-- did_crit=1 for a crit, =0 for a normal hit but it could crit
-- total = amount of hit/heal, 0 for a miss
function Recap_CreateEffect(source,effect,type,total,did_crit,is_dot)

	-- Hawksy: parameter is_dot is not currently used

	-- Hawksy: guard code
	if not effect then
		return
	end

	local typeEffect

	typeEffect = type..effect

	if (total==0 and not recap.Combatant[source].Detail[typeEffect]) then
		-- skip misses that aren't a stored effect yet, to prevent logging debuff resists
		return
	else

		if not recap.Combatant[source].Detail[typeEffect] then
			recap.Combatant[source].Detail[typeEffect] = {} -- Hits=0, HitsDmg=0, HitsMax=0, Crits=0, CritsEvents=0, CritsDmg=0, CritsMax=0, Missed=0, TicksDmg, TicksMax
		end

		if source==recap_temp.Player and not recap.Self[recap_temp.s][typeEffect] then
			recap.Self[recap_temp.s][typeEffect] = {}
		end

		if (did_crit==true) then
			-- crit
			recap.Combatant[source].Detail[typeEffect].CritsEvents = (recap.Combatant[source].Detail[typeEffect].CritsEvents or 0) + 1
			recap.Combatant[source].Detail[typeEffect].Crits = (recap.Combatant[source].Detail[typeEffect].Crits or 0) + 1
			recap.Combatant[source].Detail[typeEffect].CritsDmg = (recap.Combatant[source].Detail[typeEffect].CritsDmg or 0) + total
			recap.Combatant[source].Detail[typeEffect].CritsMax = math.max(total,recap.Combatant[source].Detail[typeEffect].CritsMax or 0)
			if source==recap_temp.Player then
				recap.Self[recap_temp.s][typeEffect].CritsEvents = (recap.Self[recap_temp.s][typeEffect].CritsEvents or 0) + 1
				recap.Self[recap_temp.s][typeEffect].Crits = (recap.Self[recap_temp.s][typeEffect].Crits or 0) + 1
				recap.Self[recap_temp.s][typeEffect].CritsDmg = (recap.Self[recap_temp.s][typeEffect].CritsDmg or 0) + total
				recap.Self[recap_temp.s][typeEffect].CritsMax = math.max(total,recap.Self[recap_temp.s][typeEffect].CritsMax or 0)
			end
		elseif (did_crit==false) then
			-- hit that could have critted (does not include misses, so that crit percentages are accurate)
			recap.Combatant[source].Detail[typeEffect].CritsEvents = (recap.Combatant[source].Detail[typeEffect].CritsEvents or 0) + 1
			recap.Combatant[source].Detail[typeEffect].Hits = (recap.Combatant[source].Detail[typeEffect].Hits or 0) + 1
			recap.Combatant[source].Detail[typeEffect].HitsDmg = (recap.Combatant[source].Detail[typeEffect].HitsDmg or 0) + total
			recap.Combatant[source].Detail[typeEffect].HitsMax = math.max(total,recap.Combatant[source].Detail[typeEffect].HitsMax or 0)
			if source==recap_temp.Player then
				recap.Self[recap_temp.s][typeEffect].CritsEvents = (recap.Self[recap_temp.s][typeEffect].CritsEvents or 0) + 1
				recap.Self[recap_temp.s][typeEffect].Hits = (recap.Self[recap_temp.s][typeEffect].Hits or 0) + 1
				recap.Self[recap_temp.s][typeEffect].HitsDmg = (recap.Self[recap_temp.s][typeEffect].HitsDmg or 0) + total
				recap.Self[recap_temp.s][typeEffect].HitsMax = math.max(total,recap.Self[recap_temp.s][typeEffect].HitsMax or 0)
			end
		elseif (total==0) then
			-- miss
			recap.Combatant[source].Detail[typeEffect].Missed = (recap.Combatant[source].Detail[typeEffect].Missed or 0) + 1
			if source==recap_temp.Player then
				recap.Self[recap_temp.s][typeEffect].Missed = (recap.Self[recap_temp.s][typeEffect].Missed or 0) + 1
			end
		else
			-- hit that couldn't have critted (must be a DoT or HoT)
			-- Ticks not stored separately, but can be calculated as (Hits+Crits-CritsEvents)
			recap.Combatant[source].Detail[typeEffect].Hits = (recap.Combatant[source].Detail[typeEffect].Hits or 0) + 1
			recap.Combatant[source].Detail[typeEffect].TicksDmg = (recap.Combatant[source].Detail[typeEffect].TicksDmg or 0) + total
			recap.Combatant[source].Detail[typeEffect].TicksMax = math.max(total,recap.Combatant[source].Detail[typeEffect].TicksMax or 0)
			if source==recap_temp.Player then
				recap.Self[recap_temp.s][typeEffect].Hits = (recap.Self[recap_temp.s][typeEffect].Hits or 0) + 1
				recap.Self[recap_temp.s][typeEffect].TicksDmg = (recap.Self[recap_temp.s][typeEffect].TicksDmg or 0) + total
				recap.Self[recap_temp.s][typeEffect].TicksMax = math.max(total,recap.Self[recap_temp.s][typeEffect].TicksMax or 0)
			end
		end
	end
end


function Recap_DeathEvent(event, info)

	local j, victim, spot
	local victimOkay

	if recap.Opt.State.value~="Stopped" then

		if recap.dump then
			local printable
			printable = CreateMessage(event, info)
			DumpMessage("DEATH;"..printable)
		end

		-- Hawksy: we often get two death events for a single death, one event with source and one without.
		-- The event with source seems to be less reliable, so we count only those without source.
		-- We occasionally get an event "with a source" but not a matching event "without a source", and
		-- when this happens Recap will miss a death.
		if (info.source == nil) then

			victim = info.victim

			-- Hawksy: if in Self Data Mode check whether the victim is the player (or players's pet)
			victimOkay = Recap_CheckSelfMode(victim, (recap.Combatant[victim] and recap.Combatant[victim].OwnedBy))

			if victimOkay then
				if info.isItem then
					-- ignore items (such as totems) that are killed
				else
					-- other deaths
					-- check for FD
					if recap.Combatant[victim] and recap.Combatant[victim].Class=="Hunter" then
						if (UnitName("player"))==victim then
							spot = "player"
						end
						for j=1,4 do
							if (UnitName("party"..j))==victim then
								spot = "party"..j
								break
							end
						end
						for j=1,40 do
							if (UnitName("raid"..j))==victim then
								spot = "raid"..j
								break
							end
						end
						if spot then
							-- Hawksy: modified to handle an indefinite number of buffs, not just 16
							j = 1
							while (UnitBuff(spot, j)) do
								if (UnitBuff(spot, j) == "Interface\\Icons\\Ability_Rogue_FeignDeath") then
									victim = nil
									break
								end
								j = j + 1
							end
						end
					end

					if victim and recap_temp.Last[victim] then
						recap_temp.Last[victim].Kills = recap_temp.Last[victim].Kills + 1
					end
				end
			end
		end
	end
end


function Recap_HealEvent(event, info)

	local source, dest, heal, effect, actualheal, overheal
	local sourceOkay, destOkay
	local event_time = GetTime()

	if recap.Opt.State.value~="Stopped" then

		if recap.dump then
			local printable
			printable = CreateMessage(event, info)
			DumpMessage("HEALING;"..printable)
		end

		if not recap.Opt.LimitFights.value then
			if recap.Opt.IdleFight.value then
				recap_temp.IdleTimer = 0
			end
			if recap.Opt.State.value=="Idle" then
				Recap_StartFight()
			end
		end

		source = info.source
		if tonumber(source) == ParserLib_SELF then
			source = recap_temp.Player
		end
		dest = info.victim
		if tonumber(dest) == ParserLib_SELF then
			dest = recap_temp.Player
		end
		-- Hawksy: there are still occasional appearances of the personal pronoun despite ParserLib
		--   (apparently only in the destination slot, but we check both)
		if recap_temp.Local.SelfPronouns[string.lower(source)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_HealEvent duplicate arg1 ("..tostring(arg1)..") event "..tostring(event).." source "..tostring(source))
if recap_temp.snap then
	Screenshot()
end
			source = recap_temp.Player
		end
		if recap_temp.Local.SelfPronouns[string.lower(dest)] then
DEFAULT_CHAT_FRAME:AddMessage("Recap_HealEvent duplicate arg1 ("..tostring(arg1)..") event "..tostring(event).." dest "..tostring(dest))
if recap_temp.snap then
	Screenshot()
end
			dest = recap_temp.Player
		end

		heal = 0
		heal = tonumber(info.amount)
		effect = info.skill
		if tonumber(effect) == ParserLib_MELEE then
			effect = "Melee"
		end
		if tonumber(effect) == ParserLib_DAMAGESHIELD then
			effect = "Damage Shield"
		end

		-- Hawksy: check for known duplicates
		if Recap_CheckForDuplicates(source, effect) then
			return
		end

		-- Hawksy: if in Self Data Mode check whether the source and dest are the player (or players's pet)
		sourceOkay = Recap_CheckSelfMode(source, (recap.Combatant[source] and recap.Combatant[source].OwnedBy))
		destOkay = Recap_CheckSelfMode(dest, (recap.Combatant[dest] and recap.Combatant[dest].OwnedBy))

		-- Hawksy: now keeping an independent timer for healing, and setting the 'WasInCurrent' flag for the healer
		--   (receiving healing still gets no registration as a combatant)
		if sourceOkay then
			Recap_CreateCombatantAndUpdateTimers(source, event_time, "Heal")
		end

		if heal and (heal > 0) then
			actualheal = heal
			if destOkay then
				if recap_temp.Last[dest] and recap_temp.Last[dest].HP then
					-- reduce 'actualheal' if we know the hit points of the target
					actualheal = math.min(recap_temp.Last[dest].MaxHP-recap_temp.Last[dest].HP,actualheal)
					recap_temp.Last[dest].HP = recap_temp.Last[dest].HP + actualheal
					if recap.debug and heal~=actualheal then
						ChatFrame2:AddMessage("|cff3fbfff"..source.."'s "..effect.." actually healed "..dest.." for "..actualheal..". ("..heal.." reported)")
					end
				end
			end

			if sourceOkay then
				-- the 'Heal' column includes only actual healing
				recap_temp.Last[source].Heal = recap_temp.Last[source].Heal + actualheal

				if not recap.Opt.LightData.value then
					-- Hawksy: the effect is credited with the full value of the heal
					-- determine whether it could have critted
					if string.find(event,"PERIODIC")~=nil then
						-- periodic event, assume not crit-capable, pass a 'nil' in the crit slot
						Recap_CreateEffect(source,effect,3,heal,nil,info.isDOT)
					else
						-- assume all others are crit capable, so pass on the value of isCrit
						if (info.isCrit == nil) then
							-- if no value for isCrit, call it false
							Recap_CreateEffect(source,effect,3,heal,false,info.isDOT)
						else
							Recap_CreateEffect(source,effect,3,heal,info.isCrit,info.isDOT)
						end
					end

					overheal = heal - actualheal
					if overheal>0 and recap.Combatant[source].Detail["3"..effect] then
						-- reuse .Missed detail to store overhealing since heals never miss
						recap.Combatant[source].Detail["3"..effect].Missed = (recap.Combatant[source].Detail["3"..effect].Missed or 0) + overheal
					end
				end
			end

			-- Hawksy: support for Recent Data Mode
			if recap.Opt.RecentData.value then
				Recap_AddRecentEvent(event_time, source, dest, 3, effect, heal, info.isCrit)
			end
		else
			-- Hawksy: no healing
			-- ignore zero healing events that have keys only from zeroEventKeys (source, victim, amount, type, skill, element)
			local i, j, allOkayIndexes
			allOkayIndexes = true
			for i in pairs(info) do
				allOkayIndexes = false
				for j=1,#zeroEventKeys do
					if i == zeroEventKeys[j] then
						-- the index is one of the okay ones
						allOkayIndexes = true
						break
					end
				end
				if not allOkayIndexes then
					-- we got through zeroEventKeys without a match for this index
					break
				end
			end
			if allOkayIndexes then
				-- probably an uninteresting event, do nothing, ignore the event
			else
				-- an unexpected event, post diagnostic information
local printable
printable = CreateMessage(event, info)
printable = "Recap_HealEvent no healing and not a miss, ignored arg1 ("..tostring(arg1)..") "..printable
DumpMessage(printable)
printable = string.gsub(printable,"\;","\ ") -- convert semicolon to blank for ordinary output
DEFAULT_CHAT_FRAME:AddMessage(printable)
if recap_temp.snap then
	Screenshot()
end
			end
		end
	end
end


-- Hawksy: ought to be obsolete with ParserLib, but keeping for a while in case
--   (might be needed to handle some non-English cases)
function Recap_CheckForDuplicates(source, effect)

	local starting, message
	local duplicate = false

	-- Hawksy: sources of "Your <spellname>" and "<critter>'s <spellname>" appear
	--   to be duplicates. They show up in the uncorrected code as combatants.
	--   They are not added to the main combatant damage. It appears to be safe
	--   to ignore these sources.  The damage these false combatants do is shown
	--   as 'melee' damage.
	-- These tests are for English, and are NOT localized since I don't know what
	--   to look for in the French and German patterns.
	-- The second English test might fail if a combatant's name or spell name
	--   contains an apostrophe (not tested yet).
	starting = string.find(source,"^Your .+")
	if starting == 1 then
		duplicate = true
	end
	starting = string.find(source,"^.+'s .+")
	if starting == 1 then
		duplicate = true
	end

	-- Hawksy: effects such as "<spellname> critically" appear to be duplicates,
	--   and it appears to be safe to ignore these effects.  Note that these
	--   duplicate effects seem to happen for healing crits only.
	-- These tests are for English and German at the moment.
	starting = string.find(effect," critically")
	if starting ~= nil then
		duplicate = true
	end
	-- German
	starting = string.find(effect," kritisch")
	if starting ~= nil then
		duplicate = true
	end

	if not duplicate then
		-- not a duplicate
		return false
	else
		-- this is a known duplicate, reject it
		message = "Recap ignoring known duplicate source or effect: ("..tostring(arg1)..") "..tostring(source).." or "..tostring(effect)
		if recap.debug or recap.dump then
			DEFAULT_CHAT_FRAME:AddMessage(message)
		end
		DumpMessage(message)
		return true
	end
end


local function Recap_ConstructSelf()

	local idx, i, j = 1
	local damage, heal = 0,0

	-- Hawksy: clear the 'SelfList' table entirely first (may cause more garbage collection, but
	--   appears to be necessary for 'table.sort' to work properly and to avoid display anomalies).
	recap_temp.SelfListSize = 1
	recap_temp.SelfList = {}

	for i in pairs(recap.Self[recap_temp.s]) do

		if not recap_temp.SelfList[idx] then
			recap_temp.SelfList[idx] = {}
		end
		recap_temp.SelfList[idx].EName = i
		recap_temp.SelfList[idx].ETotal = (recap.Self[recap_temp.s][i].HitsDmg or 0) + (recap.Self[recap_temp.s][i].TicksDmg or 0) + (recap.Self[recap_temp.s][i].CritsDmg or 0)
		recap_temp.SelfList[idx].EHits = (recap.Self[recap_temp.s][i].CritsEvents or 0) - (recap.Self[recap_temp.s][i].Crits or 0)
		recap_temp.SelfList[idx].ETicks = (recap.Self[recap_temp.s][i].Hits or 0) - recap_temp.SelfList[idx].EHits
		recap_temp.SelfList[idx].ECrits = recap.Self[recap_temp.s][i].Crits or 0
		recap_temp.SelfList[idx].EHitsMax = recap.Self[recap_temp.s][i].HitsMax or 0
		recap_temp.SelfList[idx].ETicksMax = recap.Self[recap_temp.s][i].TicksMax or 0
		recap_temp.SelfList[idx].ECritsMax = recap.Self[recap_temp.s][i].CritsMax or 0
		-- Hawksy: make the averages into numbers (not strings) to allow for sorting
		recap_temp.SelfList[idx].EHitsAvg = tonumber(string.format("%d",((recap_temp.SelfList[idx].EHits>0) and ((recap.Self[recap_temp.s][i].HitsDmg or 0)/recap_temp.SelfList[idx].EHits) or 0)))
		recap_temp.SelfList[idx].ETicksAvg = tonumber(string.format("%d",((recap_temp.SelfList[idx].ETicks>0) and ((recap.Self[recap_temp.s][i].TicksDmg or 0)/recap_temp.SelfList[idx].ETicks) or 0)))
		recap_temp.SelfList[idx].ECritsAvg = tonumber(string.format("%d",((recap_temp.SelfList[idx].ECrits>0) and ((recap.Self[recap_temp.s][i].CritsDmg or 0)/recap_temp.SelfList[idx].ECrits) or 0)))
		recap_temp.SelfList[idx].ECritsP = string.format("%.1f%%",((recap.Self[recap_temp.s][i].CritsEvents or 0)>0) and (100*recap_temp.SelfList[idx].ECrits/recap.Self[recap_temp.s][i].CritsEvents) or 0)
		recap_temp.SelfList[idx].EMaxAll = math.max( recap_temp.SelfList[idx].EHitsMax, math.max(recap_temp.SelfList[idx].ETicksMax,recap_temp.SelfList[idx].ECritsMax) )
		if tonumber(string.sub(i,1,1))<3 then -- damage event
			recap_temp.SelfList[idx].EMiss = recap.Self[recap_temp.s][i].Missed or 0
			j = recap_temp.SelfList[idx].EHits + recap_temp.SelfList[idx].ECrits + recap_temp.SelfList[idx].EMiss
			recap_temp.SelfList[idx].EMissP = string.format("%.1f%%",(j>0) and (100*(recap_temp.SelfList[idx].EMiss/j)) or 0)
			damage = damage + recap_temp.SelfList[idx].ETotal
		else
			recap_temp.SelfList[idx].EMiss = "--"
			recap_temp.SelfList[idx].EMissP = "--"
			heal = heal + recap_temp.SelfList[idx].ETotal
		end

		if recap_temp.SelfList[idx].ETicks==0 then
			recap_temp.SelfList[idx].ETicks = "--"
			recap_temp.SelfList[idx].ETicksAvg = "--"
			recap_temp.SelfList[idx].ETicksMax = "--"
		end
		if not recap.Self[recap_temp.s][i].CritsEvents then
			recap_temp.SelfList[idx].ECrits = "--"
			recap_temp.SelfList[idx].ECritsAvg = "--"
			recap_temp.SelfList[idx].ECritsMax = "--"
			recap_temp.SelfList[idx].ECritsP = "--"
			recap_temp.SelfList[idx].EHits = "--"
			recap_temp.SelfList[idx].EHitsAvg = "--"
			recap_temp.SelfList[idx].EHitsMax = "--"
		end

		idx = idx + 1
	end

	recap_temp.SelfListSize = idx

	if recap_temp.SelfListSize>1 then
		for i=1,recap_temp.SelfListSize-1 do
			if string.sub(recap_temp.SelfList[i].EName,1,1)<"3" and damage>0 then
				recap_temp.SelfList[i].ETotalP = string.format("%d%%",100*recap_temp.SelfList[i].ETotal/damage)
			elseif string.sub(recap_temp.SelfList[i].EName,1,1)>"2" and heal>0 then
				recap_temp.SelfList[i].ETotalP = string.format("%d%%",100*recap_temp.SelfList[i].ETotal/heal)
			else
				recap_temp.SelfList[i].ETotalP = "--"
			end
		end
	end

	-- Hawksy: now sort the personal details by any column
	Recap_SortSelfList()

end

function Recap_ConstructList()

	local i, j, dpsin, dpsout, entry, duration, tempname, thisCombatant
	local maxhit = 0
	local dmgin = 0
	local dmgout = 0
	local kills = 0
	local heal = 0
	local totaloverheal = 0


	if recap.Opt.SelfView.value then
		Recap_ConstructSelf()
	end

	if recap_temp.Selected~=0 then
		tempname = recap_temp.List[recap_temp.Selected].Name
	end

	if recap.Opt.View.value=="Last" then
		duration = recap[recap_temp.p].LastDuration
	else
		duration = recap[recap_temp.p].TotalDuration
	end

	-- Hawksy: clear the 'List' table entirely first (may cause more garbage collection, but
	--   appears to be necessary for 'table.sort' to work properly and to avoid display anomalies).
	recap_temp.ListSize = 1
	recap_temp.List = {}

	for i in pairs(recap.Combatant) do
		if not recap_temp.List[recap_temp.ListSize] then
			recap_temp.List[recap_temp.ListSize] = {}
		end
		entry = false

		-- Hawksy: added heals, so that healers show up from the beginning, rather than
		--   only after they have received or given damage.
		if (recap.Combatant[i].LastDmgIn==0 and recap.Combatant[i].LastDmgOut==0 and
			recap.Combatant[i].TotalDmgOut==0 and recap.Combatant[i].TotalDmgIn==0 and
			recap.Combatant[i].LastHeal==0 and recap.Combatant[i].TotalHeal==0) or
		   (recap.Opt.HideFoe.value and not recap.Combatant[i].Friend) or
		   (recap.Opt.HideYardTrash.value and (not recap.Combatant[i].Friend and recap.Combatant[i].TotalKills~=1)) or
		   (recap.Combatant[i].Ignore) then
			entry = false -- strip out unwanted combatants

		elseif recap.Opt.View.value=="Last" and recap.Combatant[i].WasInLast then
			-- Hawksy: relax time restriction on "Last Fight" stats to TotalTime rather than LastTime
			-- Hawksy: relax further, for TotalTimeIn or TotalTimeHeal also
			if not recap.Opt.HideZero.value or ((recap.Combatant[i].TotalTime>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeIn>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeHeal>recap_temp.MinTime)) then
				recap_temp.List[recap_temp.ListSize].Name = i
				recap_temp.List[recap_temp.ListSize].Time = recap.Combatant[i].LastTime
				recap_temp.List[recap_temp.ListSize].TimeIn = recap.Combatant[i].LastTimeIn
				recap_temp.List[recap_temp.ListSize].TimeHeal = recap.Combatant[i].LastTimeHeal
				recap_temp.List[recap_temp.ListSize].MaxHit = recap.Combatant[i].LastMaxHit
				recap_temp.List[recap_temp.ListSize].DmgIn = recap.Combatant[i].LastDmgIn
				recap_temp.List[recap_temp.ListSize].DmgOut = recap.Combatant[i].LastDmgOut
				recap_temp.List[recap_temp.ListSize].HPS = recap.Combatant[i].LastHPS
				recap_temp.List[recap_temp.ListSize].DPSIn = recap.Combatant[i].LastDPSIn
				recap_temp.List[recap_temp.ListSize].DPS = recap.Combatant[i].LastDPS
				recap_temp.List[recap_temp.ListSize].DPSvsAll = duration>recap_temp.MinTime and recap.Combatant[i].LastDmgOut/duration or 0
				recap_temp.List[recap_temp.ListSize].Kills = recap.Combatant[i].LastKills
				recap_temp.List[recap_temp.ListSize].Heal = recap.Combatant[i].LastHeal
				entry = true
			end

		elseif recap.Opt.View.value=="All" then
			-- Hawksy: relax further, for TotalTimeIn or TotalTimeHeal also
			if not recap.Opt.HideZero.value or ((recap.Combatant[i].TotalTime>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeIn>recap_temp.MinTime) or (recap.Combatant[i].TotalTimeHeal>recap_temp.MinTime)) then
				recap_temp.List[recap_temp.ListSize].Name = i
				recap_temp.List[recap_temp.ListSize].Time = recap.Combatant[i].TotalTime
				recap_temp.List[recap_temp.ListSize].TimeIn = recap.Combatant[i].TotalTimeIn
				recap_temp.List[recap_temp.ListSize].TimeHeal = recap.Combatant[i].TotalTimeHeal
				recap_temp.List[recap_temp.ListSize].MaxHit = recap.Combatant[i].TotalMaxHit
				recap_temp.List[recap_temp.ListSize].DmgIn = recap.Combatant[i].TotalDmgIn
				recap_temp.List[recap_temp.ListSize].DmgOut = recap.Combatant[i].TotalDmgOut
				recap_temp.List[recap_temp.ListSize].HPS = recap.Combatant[i].TotalHPS
				recap_temp.List[recap_temp.ListSize].DPSIn = recap.Combatant[i].TotalDPSIn
				recap_temp.List[recap_temp.ListSize].DPS = recap.Combatant[i].TotalDPS
				recap_temp.List[recap_temp.ListSize].DPSvsAll = duration>recap_temp.MinTime and recap.Combatant[i].TotalDmgOut/duration or 0
				recap_temp.List[recap_temp.ListSize].Kills = recap.Combatant[i].TotalKills
				recap_temp.List[recap_temp.ListSize].Heal = recap.Combatant[i].TotalHeal
				entry = true
			end
		end

		if entry then
			recap_temp.List[recap_temp.ListSize].Over = 0 -- overhealing
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[recap_temp.ListSize].Name
			if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				if recap_temp.List[recap_temp.ListSize].MaxHit > maxhit then
					maxhit = recap_temp.List[recap_temp.ListSize].MaxHit
				end
				dmgin = dmgin + recap_temp.List[recap_temp.ListSize].DmgIn
				dmgout = dmgout + recap_temp.List[recap_temp.ListSize].DmgOut
				kills = kills + recap_temp.List[recap_temp.ListSize].Kills
				heal = heal + recap_temp.List[recap_temp.ListSize].Heal
				-- Hawksy: modified to show 100% overhealing (rare case)
				if not recap.Opt.LightData.value then
					local oneheal, oneoverheal
					oneheal = recap.Combatant[i].TotalHeal
					oneoverheal = 0
					for j in pairs(recap.Combatant[i].Detail) do
						if string.sub(j,1,1)=="3" then -- a heal
							oneoverheal = oneoverheal + (recap.Combatant[i].Detail[j].Missed or 0)
						end
					end
					if ((oneheal+oneoverheal) > 0) then
						recap_temp.List[recap_temp.ListSize].Over = math.floor(((100*oneoverheal)/(oneheal+oneoverheal))+.5)
					end
					totaloverheal = totaloverheal + oneoverheal
				end
			end
			recap_temp.ListSize = recap_temp.ListSize + 1
		end
	end

	if not recap_temp.List[recap_temp.ListSize] then
		recap_temp.List[recap_temp.ListSize] = {}
	end

	-- the final entry is the totals
	recap_temp.List[recap_temp.ListSize].Name = "Totals"
	recap_temp.List[recap_temp.ListSize].Time = duration
	RecapTotals_Time:SetText(Recap_FormatTime(duration))
	recap_temp.List[recap_temp.ListSize].MaxHit = maxhit
	RecapTotals_MaxHit:SetText(maxhit)
	recap_temp.List[recap_temp.ListSize].DmgIn = dmgin
	RecapTotals_DmgIn:SetText(dmgin)
	recap_temp.List[recap_temp.ListSize].DmgOut = dmgout
	RecapTotals_DmgOut:SetText(dmgout)
	recap_temp.List[recap_temp.ListSize].Kills = kills
	RecapTotals_Kills:SetText(kills)
	recap_temp.List[recap_temp.ListSize].Heal = heal
	RecapTotals_Heal:SetText(heal)
	-- Hawksy: changed so that overhealing for zero heals shows as zero
	recap_temp.List[recap_temp.ListSize].Over = 0
	if (recap.Opt.View.value=="All") and ((heal+totaloverheal)>0) then
		recap_temp.List[recap_temp.ListSize].Over = (100*totaloverheal)/(heal+totaloverheal)
	end
	RecapTotals_Over:SetText(string.format("%d%%",recap_temp.List[recap_temp.ListSize].Over))

	-- three calculations of total friendly HPS, DPS In, and DPS
	if duration > recap_temp.MinTime then
		recap_temp.List[recap_temp.ListSize].TotHPS = heal/duration
		recap_temp.List[recap_temp.ListSize].TotDPSIn = dmgin/duration
		recap_temp.List[recap_temp.ListSize].TotDPSOut = dmgout/duration
	else
		recap_temp.List[recap_temp.ListSize].TotHPS = 0
		recap_temp.List[recap_temp.ListSize].TotDPSIn = 0
		recap_temp.List[recap_temp.ListSize].TotDPSOut = 0
	end

	RecapTotals_HPS:SetText(string.format(recap_temp.List[recap_temp.ListSize].TotHPS<1000 and "%.1f" or "%d",recap_temp.List[recap_temp.ListSize].TotHPS))
	RecapTotals_DPSIn:SetText(string.format(recap_temp.List[recap_temp.ListSize].TotDPSIn<1000 and "%.1f" or "%d",recap_temp.List[recap_temp.ListSize].TotDPSIn))
	RecapTotals_DPS:SetText(string.format(recap_temp.List[recap_temp.ListSize].TotDPSOut<1000 and "%.1f" or "%d",recap_temp.List[recap_temp.ListSize].TotDPSOut))
	RecapTotals_DPSvsAll:SetText(string.format(recap_temp.List[recap_temp.ListSize].TotDPSOut<1000 and "%.1f" or "%d",recap_temp.List[recap_temp.ListSize].TotDPSOut))

	-- Hawksy: Class column fix set to black, see also RecapScrollBar_Update and Recap_SetColumns.
	RecapTotals_Class:SetTexCoord(.9,1,.9,1)

	-- strings for when minimized
	RecapMinDPSIn_Text:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSIn))
	RecapMinDPSOut_Text:SetText(string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSOut))
	if recap.Combatant[recap_temp.Player] and recap.Opt.View.value=="Last" then
		i = recap.Combatant[recap_temp.Player].LastDPS
		if (UnitName("pet")) and (UnitName("pet"))~=UNKNOWNOBJECT and recap.Combatant[(UnitName("pet"))] then
			i = i + recap.Combatant[(UnitName("pet"))].LastDPS
		end
		RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
	elseif recap.Combatant[recap_temp.Player] and recap.Opt.View.value=="All" then
		i = recap.Combatant[recap_temp.Player].TotalDPS
		if (UnitName("pet")) and (UnitName("pet"))~=UNKNOWNOBJECT and recap.Combatant[(UnitName("pet"))] then
			i = i + recap.Combatant[(UnitName("pet"))].TotalDPS
		end
		RecapMinYourDPS_Text:SetText(string.format("%.1f",i))
	else
		RecapMinYourDPS_Text:SetText("n/a")
	end

	-- calculate percentages
	for i=1,recap_temp.ListSize-1 do
		recap_temp.List[i].HealP = 0
		recap_temp.List[i].DmgInP = 0
		recap_temp.List[i].DmgOutP = 0
		-- Hawksy: guard code
		thisCombatant = recap_temp.List[i].Name
		if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
			if recap_temp.List[recap_temp.ListSize].Heal>0 then
				recap_temp.List[i].HealP = math.floor(.5+100*recap_temp.List[i].Heal/recap_temp.List[recap_temp.ListSize].Heal)
			end
			if recap_temp.List[recap_temp.ListSize].DmgIn>0 then
				recap_temp.List[i].DmgInP = math.floor(.5+100*recap_temp.List[i].DmgIn/recap_temp.List[recap_temp.ListSize].DmgIn)
			end
			if recap_temp.List[recap_temp.ListSize].DmgOut>0 then
				recap_temp.List[i].DmgOutP = math.floor(.5+100*recap_temp.List[i].DmgOut/recap_temp.List[recap_temp.ListSize].DmgOut)
			end
		end
	end

	-- restore a valid value for Selected if possible
	if tempname then -- follow name to pre-sort selected
		recap_temp.Selected = Recap_GetSelectedByName(tempname)
		if recap_temp.Selected~=0 then
			RecapPanel_Show(tempname)
		else
			RecapPanel_Hide()
		end
	end

	Recap_SortList()

	RecapScrollBar_Update()
	Recap_UpdatePlugins()

end

function Recap_DefineGauges()

	local i, thisCombatant

	recap_temp.GaugeMax = 0

	if recap_temp.ListSize>1 then
		recap_temp.GaugeBy = string.gsub(recap.Opt.SortBy.value,"P$","")
		recap_temp.GaugeBy = recap_temp.GaugeBy=="DPSvsAll" and "DmgOut" or recap_temp.GaugeBy
		if not gaugecolor[recap_temp.GaugeBy] then
			recap_temp.GaugeBy = nil
		else
			for i=1,recap_temp.ListSize-1 do
				-- Hawksy: guard code
				thisCombatant = recap_temp.List[i].Name
				if thisCombatant and recap.Combatant[thisCombatant] and recap_temp.GaugeBy then
					recap_temp.GaugeMax = math.max(recap.Combatant[thisCombatant].Friend and recap_temp.List[i][recap_temp.GaugeBy] or 0,recap_temp.GaugeMax)
				end
			end
			for i=1,15 do
				getglobal("RecapList"..i.."_Gauge"):SetVertexColor(gaugecolor[recap_temp.GaugeBy].r,gaugecolor[recap_temp.GaugeBy].g,gaugecolor[recap_temp.GaugeBy].b)
			end
		end
	end
end


function Recap_GetSelectedByName(arg1)

	local sel, i = 0
	local thisCombatant

	if arg1 and recap_temp.ListSize>1 then
		for i=1,recap_temp.ListSize-1 do
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[i].Name
			if thisCombatant and (thisCombatant==arg1) then
				sel = i
			end
		end
	end

	return sel
end

--[[ Sorting functions ]]

-- initial sort by field done with table.sort for speed, then
-- friends shifted to top with an insertion sort for "stable" list
function Recap_SortList()

	local temp, tempname

	if recap_temp.Selected~=0 then
		tempname = recap_temp.List[recap_temp.Selected].Name
	end

	-- remember the last ('Total') element
	temp = recap_temp.List[recap_temp.ListSize]

	-- Hawksy: remove the last element from the 'List' table in case it could
	--   confuse 'table.sort'.
	table.remove(recap_temp.List) -- without second parameter defaults to last element

	if recap.Opt.SortBy.value then
		if recap.Opt.SortDir.value then
			table.sort(recap_temp.List,Recap_SortDown)
		else
			table.sort(recap_temp.List,Recap_SortUp)
		end
		Recap_SortFriends()
	end

	-- restore the last ('Total') element
	recap_temp.List[recap_temp.ListSize] = temp

	Recap_DefineGauges()
	recap_temp.Selected = Recap_GetSelectedByName(tempname)

end

function Recap_SortDown(e1,e2)

	-- Hawksy: guard code
	if e1[recap.Opt.SortBy.value] and e2[recap.Opt.SortBy.value] then
		if ( e1[recap.Opt.SortBy.value] < e2[recap.Opt.SortBy.value] ) then
			return true
		else
			return false
		end
	else
		return false
	end

end

function Recap_SortUp(e1,e2)

	-- Hawksy: guard code
	if e1[recap.Opt.SortBy.value] and e2[recap.Opt.SortBy.value] then
		if ( e1[recap.Opt.SortBy.value] > e2[recap.Opt.SortBy.value] ) then
			return true
		else
			return false
		end
	else
		return false
	end

end

-- perform stable insertion sort on the list
function Recap_SortFriends()

	local i, j, temp, other
	local changed = true

	if recap_temp.ListSize>2 then
		for i=2,(recap_temp.ListSize-1) do
			temp = recap_temp.List[i]
			-- Hawksy: guard code
			if (temp) and (temp.Name) and (recap.Combatant[temp.Name]) and (recap.Combatant[temp.Name].Friend) then
				j=i
				while (j>1) do
					other = recap_temp.List[j-1]
					-- Hawksy: guard code
					if (other) and (other.Name) and (recap.Combatant[other.Name]) and (not recap.Combatant[other.Name].Friend) then
						recap_temp.List[j] = other
						j = j - 1
					else
						break
					end
				end
				recap_temp.List[j] = temp
			end
		end
	end

end

-- Hawksy: three new functions allowing the personal details ("Self") panel to be sorted
--   by column (damage effects always appear first, healing effects second) (function
--   Recap_SelfDamageSort, which was a fixed sort, has been removed)
function Recap_SortSelfList()

	if recap.Opt.ESortBy.value then
		if recap.Opt.ESortDir.value then
			table.sort(recap_temp.SelfList,Recap_SortSelfDown)
		else
			table.sort(recap_temp.SelfList,Recap_SortSelfUp)
		end
	end

end

function Recap_SortSelfDown(e1,e2)

	local effect1, effect2, value1, value2

	if e1.EName and e2.EName then
		effect1 = string.sub(e1.EName,1,1)
		effect2 = string.sub(e2.EName,1,1)
		if e1[recap.Opt.ESortBy.value] and e2[recap.Opt.ESortBy.value] then
			value1 = e1[recap.Opt.ESortBy.value]
			if value1 == "--" then
				value1 = -1
			end
			-- turn a percentage string into a number
			if (type(value1) == "string") and (string.sub(value1, -1) == "%") then
				value1 = tonumber(string.sub(value1, 1, string.len(value1)-1))
			end
			value2 = e2[recap.Opt.ESortBy.value]
			if value2 == "--" then
				value2 = -1
			end
			-- turn a percentage string into a number
			if (type(value2) == "string") and (string.sub(value2, -1) == "%") then
				value2 = tonumber(string.sub(value2, 1, string.len(value2)-1))
			end
			if (((value1 < value2) and (effect1==effect2)) or (effect1<effect2)) then
				return true
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end

end

function Recap_SortSelfUp(e1,e2)

	local effect1, effect2, value1, value2

	if e1.EName and e2.EName then
		effect1 = string.sub(e1.EName,1,1)
		effect2 = string.sub(e2.EName,1,1)
		if e1[recap.Opt.ESortBy.value] and e2[recap.Opt.ESortBy.value] then
			value1 = e1[recap.Opt.ESortBy.value]
			if value1 == "--" then
				value1 = -1
			end
			-- turn a percentage string into a number
			if (type(value1) == "string") and (string.sub(value1, -1) == "%") then
				value1 = tonumber(string.sub(value1, 1, string.len(value1)-1))
			end
			value2 = e2[recap.Opt.ESortBy.value]
			if value2 == "--" then
				value2 = -1
			end
			-- turn a percentage string into a number
			if (type(value2) == "string") and (string.sub(value2, -1) == "%") then
				value2 = tonumber(string.sub(value2, 1, string.len(value2)-1))
			end
			if (((value1 > value2) and (effect1==effect2)) or (effect1<effect2)) then
				return true
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end

end

--[[ Window movement functions ]]

function Recap_OnMouseDown(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" and not recap.Opt.Pinned.value then
		RecapFrame:StartMoving()
	end
end

function Recap_OnMouseUp(arg1)

	if recap_temp.Loaded and arg1=="LeftButton" and not recap.Opt.Pinned.value then
		RecapFrame:StopMovingOrSizing()
		Recap_CheckWindowBounds()

	elseif arg1=="RightButton" and recap.Opt.Minimized.value then
		RecapCreateMenu(recap_temp.MinMenu, nil)
	end

end

-- repositions RecapFrame if any part of it goes off the screen
function Recap_CheckWindowBounds()

	local x1, y1, x2, y2, cx, cy
	local needs_moved = false

	if recap_temp.Loaded and RecapFrame then

		x1 = RecapFrame:GetLeft()
		y1 = RecapFrame:GetTop()
		cx = RecapFrame:GetWidth()
		cy = RecapFrame:GetHeight()

		if x1 and y1 and cx and cy then

			x2 = RecapFrame:GetRight()
			y2 = RecapFrame:GetBottom()
			uix2 = UIParent:GetRight()
			uix1 = UIParent:GetLeft()
			uiy1 = UIParent:GetTop()
			uiy2 = UIParent:GetBottom()

			if x2 and uix2 and x2 > uix2 then
				x1 = x1 + uix2 - x2
				needs_moved = true
			elseif uix1 and uix1 > x1 then
				x1 = x1 + uix1 - x1
				needs_moved = true
			end

			if y1 and y1 > uiy1 then
				y1 = y1 + uiy1 - y1
				needs_moved = true
			elseif uiy2 and uiy2 > y2 then
				y1 = y1 + uiy2 - y2
				needs_moved = true
			end

			if needs_moved then
				RecapFrame:ClearAllPoints()
				RecapFrame:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",x1,y1)
			end

			recap[recap_temp.p].WindowTop = RecapFrame:GetTop()
			recap[recap_temp.p].WindowLeft = RecapFrame:GetLeft()
			recap[recap_temp.p].WindowHeight = RecapFrame:GetHeight()
			recap[recap_temp.p].WindowWidth = RecapFrame:GetWidth()

		end

	end
end

--[[ Dialog control functions ]]

function Recap_OnClick(arg1) -- clicks on buttons

	local i

	if not recap_temp.Loaded then
		return
	end

	local fight_count = 0
	local total_fight_count = 0
	local filen = RecapSetEditBox:GetText()
	local set_friend, set_dmgin, set_dmgout, set_maxhit, set_kills, set_heal, set_time

	PlaySound("GAMEGENERICBUTTONPRESS")

	if arg1=="Close" then
		RecapFrame_Hide()

	elseif arg1=="Minimize" then

		if recap.Opt.Minimized.value then
			Recap_Maximize()
		else
			Recap_Minimize()
		end
		RecapFrame:SetAlpha(1)
		recap_temp.FadeTimer = -1
		Recap_SetButtons()
		Recap_OnTooltip("Minimize")

	elseif arg1=="Pin" then
		recap.Opt.Pinned.value = not recap.Opt.Pinned.value
		Recap_SetButtons()
		Recap_OnTooltip("Pin")

	elseif arg1=="Pause" then
		if recap.Opt.Paused.value then
			recap.Opt.Paused.value = false
			Recap_SetState("Idle")
		else
			recap.Opt.Paused.value = true
			Recap_EndFight()
			Recap_SetState("Stopped")
		end
		Recap_SetButtons()
		Recap_OnTooltip("Pause")

	elseif arg1=="ShowAll" then
		if recap.Opt.SelfView.value then
			recap.Opt.SelfView.value = false
			if not recap.Opt.Minimized.value then
				Recap_SetColumns()
			end
		end

		if recap.Opt.View.value=="Last" then
			Recap_SetView("All")
		else
			Recap_SetView("Last")
		end

		Recap_OnTooltip("ShowAll")

	elseif arg1=="Options" then
		if RecapOptFrame:IsShown() then
			RecapOptFrame:Hide()
		else
			RecapOptFrame:Show()
		end

	elseif arg1=="Reset" then

		if recap.Opt.SelfView.value then
			recap.Self[recap_temp.s] = {}
			Recap_SetView()

		elseif recap.Opt.View.value=="All" then
			-- wipe everything --
			RecapPanel_Hide(1)
			Recap_ResetAllCombatants(false)
			Recap_MakeFriends()
			Recap_SetView()

		elseif recap.Opt.View.value=="Last" then
			-- Hawksy: add a confirm dialog
			StaticPopupDialogs["RECAP_CONFIRMLASTRESET"] = {
				text = TEXT(recap_temp.Local.ConfirmLastReset),
				button1 = TEXT(ACCEPT),
				button2 = TEXT(CANCEL),
				OnAccept = function()
					-- SUBTRACT the last fight --
					for i in pairs(recap.Combatant) do
						if recap.Combatant[i].WasInLast then
							Recap_ResetLastFight(i)
						end
					end
					recap[recap_temp.p].TotalDuration = recap[recap_temp.p].TotalDuration - recap[recap_temp.p].LastDuration
					recap[recap_temp.p].LastDuration = 0
					Recap_SetView()
				end,
				OnHide = function()
				end,
				OnCancel = function()
				end,
				timeout = 0,
				showAlert = 1,
				whileDead = 1
			}
			StaticPopup_Show("RECAP_CONFIRMLASTRESET")
		end

	elseif arg1=="SaveSet" then
		RecapSetEditBox:ClearFocus()
		if filen and filen~="" then
			filen = string.gsub(filen,"\"","'") -- convert "s to 's
			filen = string.gsub(filen,"%[","(") -- convert [s to (s
			filen = string.gsub(filen,"%]",")") -- convert ]s to )s
		end
		RecapSetEditBox:SetText("")
		Recap_SaveCombatants(filen,recap.Opt.ReplaceSet.value)
		Recap_BuildFightSets()
		recap_temp.FightSetSelected = 0
		RecapFightSetsScrollBar_Update()

	elseif arg1=="LoadSet" then
		RecapSetEditBox:SetText("")
		RecapSetEditBox:ClearFocus()
		Recap_LoadCombatants(filen,recap.Opt.ReplaceSet.value)
		Recap_MakeFriends()
		Recap_SetView("All")

	elseif arg1=="DeleteSet" then
		RecapSetEditBox:SetText("")
		RecapSetEditBox:ClearFocus()
		recap_set[filen] = nil
		Recap_BuildFightSets()

	elseif arg1=="SelfView" then
		recap.Opt.SelfView.value = not recap.Opt.SelfView.value
		RecapPanel_Hide(1)
		Recap_SetColumns()
		Recap_OnTooltip("SelfView")
	end
end

-- removes combatant i from last fight
function Recap_ResetLastFight(i)
	recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime - recap.Combatant[i].LastTime
	recap.Combatant[i].TotalTimeIn = recap.Combatant[i].TotalTimeIn - recap.Combatant[i].LastTimeIn
	recap.Combatant[i].TotalTimeHeal = recap.Combatant[i].TotalTimeHeal - recap.Combatant[i].LastTimeHeal
	recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn - recap.Combatant[i].LastDmgIn
	recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut - recap.Combatant[i].LastDmgOut
	recap.Combatant[i].TotalHeal = recap.Combatant[i].TotalHeal - recap.Combatant[i].LastHeal
	recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills - recap.Combatant[i].LastKills
	if recap.Combatant[i].TotalTime > recap_temp.MinTime then
		recap.Combatant[i].TotalDPS = recap.Combatant[i].TotalDmgOut/recap.Combatant[i].TotalTime
	else
		recap.Combatant[i].TotalDPS = 0
	end
	if recap.Combatant[i].TotalTimeIn > recap_temp.MinTime then
		recap.Combatant[i].TotalDPSIn = recap.Combatant[i].TotalDmgIn/recap.Combatant[i].TotalTimeIn
	else
		recap.Combatant[i].TotalDPSIn = 0
	end
	if recap.Combatant[i].TotalTimeHeal > recap_temp.MinTime then
		recap.Combatant[i].TotalHPS = recap.Combatant[i].TotalHeal/recap.Combatant[i].TotalTimeHeal
	else
		recap.Combatant[i].TotalHPS = 0
	end
	recap.Combatant[i].WasInLast = false
	recap.Combatant[i].LastTime = 0
	recap.Combatant[i].LastTimeIn = 0
	recap.Combatant[i].LastTimeHeal = 0
	recap.Combatant[i].LastDmgIn = 0
	recap.Combatant[i].LastDmgOut = 0
	recap.Combatant[i].LastDPS = 0
	recap.Combatant[i].LastDPSIn = 0
	recap.Combatant[i].LastHPS = 0
	recap.Combatant[i].LastMaxHit = 0
	recap.Combatant[i].LastKills = 0
	recap.Combatant[i].LastHeal = 0
end

function RecapHeader_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local msg=string.gsub(this:GetName(),"RecapHeader_","Header")
	Recap_OnTooltip(msg)

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.ColumnMenu, nil)
	end
end

-- sort column, toggle dir for new headers, name=ascending, rest=descending
function RecapHeader_OnClick()

	local text=""
	local channel = "" -- "Guild" "Say" etc or "Channel"
	local chatnumber = 0 -- if channel is "Channel", this is the channel numbers
	local arg1= string.gsub(this:GetName(),"RecapHeader_","")

	if not IsShiftKeyDown() then
		if recap.Opt.SortBy.value==string.gsub(arg1,"P$","") then
			recap.Opt.SortDir.value = not recap.Opt.SortDir.value
		else
			recap.Opt.SortBy.value = string.gsub(arg1,"P$","")

			if arg1=="Name" then
				recap.Opt.SortDir.value = true
			else
				recap.Opt.SortDir.value = false
			end
		end
		Recap_SortList()
		RecapScrollBar_Update()

	elseif IsShiftKeyDown() and ChatFrameEditBox:IsVisible() and recap_temp.ListSize>1 and arg1~="Name" then

		recap.Opt.SortBy.value = arg1
		recap.Opt.SortDir.value = false
		Recap_SortList()
		RecapScrollBar_Update()

		if recap.Opt.SpamRows.value then
			channel = ChatFrameEditBox:GetAttribute("chatType")
			chatnumber = nil
			if channel=="WHISPER" then
				chatnumber = ChatFrameEditBox:GetAttribute("tellTarget")
			elseif channel=="CHANNEL" then
				chatnumber = ChatFrameEditBox:GetAttribute("channelTarget")
			end
			Recap_PostSpamRows(arg1, channel, chatnumber)
		else
			Recap_InsertChat(Recap_PostSpamLine(arg1))
		end

	-- Hawksy: new post to "Self"
	elseif IsShiftKeyDown() and recap_temp.ListSize>1 and arg1~="Name" then

		recap.Opt.SortBy.value = arg1
		recap.Opt.SortDir.value = false
		Recap_SortList()
		RecapScrollBar_Update()

		if recap.Opt.SpamRows.value then
			-- post to "Self"
			Recap_PostSpamRows(arg1, nil, nil)
		else
			DEFAULT_CHAT_FRAME:AddMessage(Recap_PostSpamLine(arg1))
		end

	elseif IsShiftKeyDown() and not ChatFrameEditBox:IsVisible() then
		-- Hawksy: will now happen seldom
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Local.RankUsage)
	end

end

function Recap_List_OnClick(arg1)

	local id, index, thisCombatant
	local text = ""
	local channel = "" -- "Guild" "Say" etc or "Channel"

	id = this:GetID()
	index = id + FauxScrollFrame_GetOffset(RecapScrollBar)

	if index<recap_temp.ListSize then
		recap_temp.DetailSelected = 0
		if recap_temp.Selected==index and arg1=="LeftButton" then
			recap_temp.Selected = 0
			getglobal("RecapList"..id):UnlockHighlight()
			RecapPanel_Hide()
		else
			-- Hawksy: guard code
			thisCombatant = recap_temp.List[index].Name
			if thisCombatant then
				recap_temp.Selected = index
				RecapScrollBar_Update()
				RecapPanel_Show(thisCombatant)
			else
				recap_temp.Selected = 0
				getglobal("RecapList"..id):UnlockHighlight()
				RecapPanel_Hide()
			end
		end
	end

	if arg1=="RightButton" and index<recap_temp.ListSize and IsShiftKeyDown() and not IsControlKeyDown() and ChatFrameEditBox:IsVisible() then
		-- Hawksy: modified to post Recent Incoming events when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			channel = ChatFrameEditBox:GetAttribute("chatType")
			Recap_PostRecentRows(false, recap_temp.List[index].Name, nil, channel)
		else
			ChatFrameEditBox:Insert(recap_temp.List[index].Name.." "..getglobal("RecapList"..id.."_DPS"):GetText().." DPS ")
		end

	elseif arg1=="RightButton" and index<recap_temp.ListSize and IsShiftKeyDown() and not IsControlKeyDown() then
		-- Hawksy: post Recent Incoming events to Self channel if no chat edit box is visible
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(false, recap_temp.List[index].Name, nil, "Self")
		end

	elseif arg1=="RightButton" and index<recap_temp.ListSize and IsControlKeyDown() and not IsShiftKeyDown() and ChatFrameEditBox:IsVisible() then
		-- Hawksy: modified to post Recent Outgoing events when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			channel = ChatFrameEditBox:GetAttribute("chatType")
			Recap_PostRecentRows(true, recap_temp.List[index].Name, nil, channel)
		else
			ChatFrameEditBox:Insert(recap_temp.List[index].Name.." "..getglobal("RecapList"..id.."_DPS"):GetText().." DPS ")
		end

	elseif arg1=="RightButton" and index<recap_temp.ListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Outgoing events to Self channel if no chat edit box is visible
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, recap_temp.List[index].Name, nil, "Self")
		end

	-- Hawksy: control+shift+right-click on any line posts all recent combat event information
	-- to avoid impossible spam, this function only posts to "Self" (the player console)
	-- with a lot of events, it may be slow
	-- this feature is deliberately undocumented
	elseif arg1=="RightButton" and IsShiftKeyDown() and IsControlKeyDown() then
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, nil, nil, "Self")
		end

	elseif arg1=="LeftButton" and index<recap_temp.ListSize and IsShiftKeyDown() and ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Insert(string.format(recap_temp.Local.VerboseLinkStart,
								recap_temp.List[index].Name,
								getglobal("RecapList"..id.."_DmgIn"):GetText(),
								getglobal("RecapList"..id.."_DPSIn"):GetText(),
								getglobal("RecapList"..id.."_DmgOut"):GetText(),
								getglobal("RecapList"..id.."_Time"):GetText(),
								getglobal("RecapList"..id.."_DPS"):GetText(),
								getglobal("RecapList"..id.."_MaxHit"):GetText(),
								getglobal("RecapList"..id.."_Heal"):GetText(),
								getglobal("RecapList"..id.."_HPS"):GetText() ).." for "..
								recap_temp.Local.LastAll[recap.Opt.View.value])

	-- Hawksy: new post to "Self"
	elseif arg1=="LeftButton" and index<recap_temp.ListSize and IsShiftKeyDown() then
		DEFAULT_CHAT_FRAME:AddMessage(string.format(recap_temp.Local.VerboseLinkStart,
										recap_temp.List[index].Name,
										getglobal("RecapList"..id.."_DmgIn"):GetText(),
										getglobal("RecapList"..id.."_DPSIn"):GetText(),
										getglobal("RecapList"..id.."_DmgOut"):GetText(),
										getglobal("RecapList"..id.."_Time"):GetText(),
										getglobal("RecapList"..id.."_DPS"):GetText(),
										getglobal("RecapList"..id.."_MaxHit"):GetText(),
										getglobal("RecapList"..id.."_Heal"):GetText(),
										getglobal("RecapList"..id.."_HPS"):GetText() ).." for "..
										recap_temp.Local.LastAll[recap.Opt.View.value])

	elseif arg1=="RightButton" and index<recap_temp.ListSize and not IsShiftKeyDown() and not IsControlKeyDown() then
		-- Hawksy: guard code
		thisCombatant = recap_temp.List[index].Name
		if thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
			RecapCreateMenu(recap_temp.DropMenu, 1)
		else
			RecapCreateMenu(recap_temp.AddMenu, 1)
		end
	end
end

-- Hawksy: two functions to support Recent Data Mode
function Recap_CreateRecent()

	local i
	local current_state = recap.Opt.State.value
	recap.Opt.State.value = "Stopped"

	recap_temp.Recent = {}
	i = 1
	while i <= recap.Opt.RecentEventCount.value do
		recap_temp.Recent[i] = {}
		i = i + 1
	end
	recap_temp.Recent.First = 1
	recap_temp.Recent.Last = 1

	recap.Opt.State.value = current_state
end

function Recap_AddRecentEvent(event_time, source, dest, effectType, effect, amount, crit)

	local i

	-- play with the pointers
	if recap_temp.Recent[recap_temp.Recent.Last] and recap_temp.Recent[recap_temp.Recent.Last].T then
		-- events already stored, move Last pointer
		if recap_temp.Recent.Last >= recap.Opt.RecentEventCount.value then
			recap_temp.Recent.Last = 1
		else
			recap_temp.Recent.Last = recap_temp.Recent.Last + 1
		end
		if recap_temp.Recent.First == recap_temp.Recent.Last then
			-- pointers have met, move the First pointer out of the way
			if recap_temp.Recent.First >= recap.Opt.RecentEventCount.value then
				recap_temp.Recent.First = 1
			else
				recap_temp.Recent.First = recap_temp.Recent.First + 1
			end
		end
	else
		-- no event stored yet, store at Last
	end
	i = recap_temp.Recent.Last
	-- single character keys to keep memory cost down (T = time, S = source D = destination, E = effect, A = amount, C = crit)
	recap_temp.Recent[i].T = event_time
	if source then
		recap_temp.Recent[i].S = source
		recap_temp.Recent[i].D = dest
		recap_temp.Recent[i].E = tostring(effectType)..effect
		recap_temp.Recent[i].A = amount
		recap_temp.Recent[i].C = crit
	else
		-- if source is nil, this represents an 'end of fight' event
		recap_temp.Recent[i].S = "**EOF**"
		recap_temp.Recent[i].D = nil
		recap_temp.Recent[i].E = nil
		recap_temp.Recent[i].A = nil
		recap_temp.Recent[i].C = nil
	end
end

--[[ Tooltip functions ]]

function Recap_Tooltip(arg1,arg2)

	if recap_temp.Loaded and recap.Opt.ShowTooltips.value then
		if not recap.Opt.TooltipFollow.value then
			GameTooltip_SetDefaultAnchor(GameTooltip,this)
		else
			GameTooltip:SetOwner(this,Recap_TooltipAnchor())
		end
		GameTooltip:SetText(arg1)
		GameTooltip:AddLine(arg2, .75, .75, .75, 1)
		GameTooltip:Show()
	end
end

-- returns line1,line2 of a tooltip for arg1 in OptList (for generic static tooltips)
function Recap_GetTooltip(arg1)

	local i

	if arg1 then
		for i in pairs(recap_temp.OptList) do
			if recap_temp.OptList[i][1]==arg1 then
				return recap_temp.OptList[i][2],recap_temp.OptList[i][3]
			end
		end
	end
end

-- arg1=tooltip index, arg2=optional addition to header
function Recap_OnTooltip(arg1,arg2)

	local line1, line2

	if arg1=="Status" then
		Recap_Tooltip("Recap Status: "..recap.Opt.State.value,recap_temp.Local.StatusTooltip)
	elseif arg1=="Close" then
		if recap.Opt.State.value=="Stopped" then
			Recap_OnTooltip("ExitRecap")
		else
			Recap_OnTooltip("HideWindow")
		end
	elseif arg1=="Minimize" then
		if recap.Opt.Minimized.value then
			Recap_OnTooltip("ExpandWindow")
		else
			Recap_OnTooltip("MinimizeWindow")
		end
	elseif arg1=="Pin" then
		if recap.Opt.Pinned.value then
			Recap_OnTooltip("UnPinWindow")
		else
			Recap_OnTooltip("PinWindow")
		end
	elseif arg1=="Pause" then
		if recap.Opt.State.value=="Stopped" then
			Recap_OnTooltip("Resume")
		else
			Recap_OnTooltip("PauseMonitoring")
		end
	elseif arg1=="ShowAll" then
		if recap.Opt.View.value=="Last" then
			Recap_OnTooltip("ShowAllFights")
		else
			Recap_OnTooltip("ShowLastFight")
		end
	elseif arg1=="HeaderName" then
		if recap.Opt.View.value=="Last" then
			Recap_OnTooltip("CombatLast")
		else
			Recap_OnTooltip("CombatAll")
		end
	elseif arg1=="Reset" then
		if recap.Opt.SelfView.value then
			Recap_OnTooltip("ResetSelfView")
		elseif recap.Opt.View.value=="Last" then
			Recap_OnTooltip("ResetLastFight")
		else
			Recap_OnTooltip("ResetAllTotals")
		end
	elseif arg1=="SelfView" then
		if recap.Opt.SelfView.value then
			Recap_OnTooltip("HideSelfView")
		else
			Recap_OnTooltip("ShowSelfView")
		end
	else
		line1,line2 = Recap_GetTooltip(arg1)
		if line1 and line2 then
			if arg2 then
				Recap_Tooltip(line1..": "..arg2,line2)
			else
				Recap_Tooltip(line1, line2)
			end
		end
	end
end

function Recap_List_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local index, id

	id = this:GetID()
	index = id + FauxScrollFrame_GetOffset(RecapScrollBar)

	-- Hawksy: altered from 110 to 130 in hopes to match change of Name width from 124 to 144,
	--   not entirely sure that it is the right thing to do.
	thisCombatant = recap_temp.List[index].Name
	if getglobal("RecapList"..id.."_Name"):GetStringWidth()>130 then
		if thisCombatant then
			Recap_Tooltip(thisCombatant)
		end
	end
	if (index<recap_temp.ListSize) and recap_temp.Selected==0 then
		if thisCombatant then
			RecapPanel_Show(thisCombatant)
		end
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function Recap_Totals_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local i, thisCombatant
	local r = recap_temp.ColorWhite.r
	local g = recap_temp.ColorWhite.g
	local b = recap_temp.ColorWhite.b
	local other_kills = 0

	if recap.Opt.ShowTooltips.value then

		if recap.Opt.TooltipFollow.value then
			GameTooltip:SetOwner(this,Recap_TooltipAnchor())
		else
			GameTooltip_SetDefaultAnchor(GameTooltip,this)
		end

		GameTooltip:AddLine("Totals for "..recap_temp.Local.LastAll[recap.Opt.View.value])

		if recap_temp.ListSize>1 then

			for i=1,(recap_temp.ListSize-1) do
				-- Hawksy: guard code
				thisCombatant = recap_temp.List[i].Name
				if thisCombatant and recap.Combatant[thisCombatant] and (not recap.Combatant[thisCombatant].Friend) then
					other_kills = other_kills + recap_temp.List[i].Kills
				end
			end
			GameTooltip:AddDoubleLine("Combatants:",recap_temp.ListSize-1,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine("Time Fighting:",Recap_FormatTime(recap_temp.List[recap_temp.ListSize].Time),r,g,b,r,g,b)
			GameTooltip:AddDoubleLine("Max Hit:",recap_temp.List[recap_temp.ListSize].MaxHit,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine("Deaths:",recap_temp.List[recap_temp.ListSize].Kills,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine("Kills:",other_kills,r,g,b,r,g,b)
			GameTooltip:AddDoubleLine("Heals:",recap_temp.List[recap_temp.ListSize].Heal,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			GameTooltip:AddDoubleLine("HPS Out:",string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotHPS),recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b,recap_temp.ColorHeal.r,recap_temp.ColorHeal.g,recap_temp.ColorHeal.b)
			GameTooltip:AddDoubleLine("Damage In:",recap_temp.List[recap_temp.ListSize].DmgIn,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine("DPS In:",string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSIn),recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b,recap_temp.ColorDmgIn.r,recap_temp.ColorDmgIn.g,recap_temp.ColorDmgIn.b)
			GameTooltip:AddDoubleLine("Damage Out:",recap_temp.List[recap_temp.ListSize].DmgOut,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
			GameTooltip:AddDoubleLine("DPS Out:",string.format("%.1f",recap_temp.List[recap_temp.ListSize].TotDPSOut),recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b,recap_temp.ColorDmgOut.r,recap_temp.ColorDmgOut.g,recap_temp.ColorDmgOut.b)
		else
			GameTooltip:AddLine("Combatants: none",r,g,b)
		end

		GameTooltip:Show()
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function Recap_TooltipAnchor()

	local anchor = "ANCHOR_LEFT"

	if GetCursorPosition("UIParent")<(UIParent:GetWidth()/2) then
		anchor = "ANCHOR_RIGHT"
	end
	return anchor
end

--[[ Plug-in functions ]]

function Recap_GetIB_Tooltip()
	return recap_temp.Local.LastAll[recap.Opt.View.value]..":\nYour DPS: "..RecapMinYourDPS_Text:GetText().."\nMax hit: "..RecapTotals_MaxHit:GetText().."\nTotal DPS Out: "..RecapTotals_DPS:GetText().."\nTotal DPS In: "..RecapTotals_TotDPSIn:GetText()
end

function Recap_UpdatePlugins()

	local yourdps = RecapMinYourDPS_Text:GetText()
	local dpsin = RecapMinDPSIn_Text:GetText()
	local dpsout = RecapMinDPSOut_Text:GetText()

	if IB_Recap_Update then
		IB_Recap_Update("DPS: "..yourdps)
	end

	if TitanPanelRecap_Update then
		TitanPanelRecap_Update(recap.Opt.State.value,yourdps,dpsin,dpsout)
	end
end

--[[ Misc functions ]]

-- if this is a pet, pass owner's name to ownedby
function Recap_AddFriend(thisCombatant, ownedby)

	if thisCombatant and thisCombatant~=UNKNOWNOBJECT and recap.Combatant[thisCombatant] then
		recap.Combatant[thisCombatant].Friend = true
		if ownedby then
			-- pet could potentially do damage without owner, add owner if so
			if not recap.Combatant[ownedby] then
				Recap_CreateBlankCombatant(ownedby)
				recap.Combatant[ownedby].Friend = true
			end
			if not recap_temp.Last[ownedby] then
				recap_temp.Last[ownedby] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, Heal = 0 }
			end
			recap.Combatant[thisCombatant].OwnedBy = ownedby
			recap.Combatant[ownedby].OwnsPet = thisCombatant
		end
	end
end

function Recap_MakeFriends()

	local i, item, u

	Recap_GetUnitInfo("player")
	Recap_AddFriend(recap_temp.Player, nil)
	if UnitExists("pet") then
		Recap_GetUnitInfo("pet")
		Recap_AddFriend((UnitName("pet")), (UnitName("player")))
	end

	-- ignore party and raid members if in Self Data Mode
	if not recap.Opt.SelfData.value then
		for i=1,4 do
			if UnitExists("party"..i) then
				Recap_GetUnitInfo("party"..i)
				-- Hawksy: fix here and everywhere else UnitName is called.
				--   Since patch 1.12 UnitName returns two values, name and realm;
				--   and if included as an argument will insert two arguments into
				--   the call, which for this particular call would make the combatant
				--   appear to be the pet of their realm (!).  This has caused problems
				--   in battlegrounds.  The fix is to force UnitName to return one value
				--   only, which is done by wrapping it in parentheses in the context of
				--   a call.  I've done the wrapping everywhere, whether necessary or not.
				Recap_AddFriend((UnitName("party"..i)), nil)
				if UnitExists("partypet"..i) then
					Recap_GetUnitInfo("partypet"..i)
					Recap_AddFriend((UnitName("partypet"..i)), (UnitName("party"..i)))
				end
			end
		end

		if (GetNumRaidMembers()>0) then
			for i=1,40 do
				if UnitExists("raid"..i) then
					Recap_GetUnitInfo("raid"..i)
					Recap_AddFriend((UnitName("raid"..i)), nil)
					if UnitExists("raidpet"..i) then
						Recap_GetUnitInfo("raidpet"..i)
						Recap_AddFriend((UnitName("raidpet"..i)), (UnitName("raid"..i)))
					end
				end
			end
		end
	end
end

-- returns a string of seconds converted to 0:00:00 format
function Recap_FormatTime(arg1)

	local hours, minutes, seconds
	local text

	seconds = math.floor(arg1+0.5)
	hours = math.floor(seconds/3600)
	seconds = seconds - hours*3600
	minutes = math.floor(seconds/60)
	seconds = seconds - minutes*60

	return ((hours>0) and (hours..":") or "") .. string.format("%02d:",minutes) .. string.format("%02d",seconds)
end

-- returns a string of seconds converted to 00:00.0 format (nearest tenth of a second, hours not included)
function Recap_FormatTimeRecent(arg1)

	local hours, minutes, seconds, deciseconds
	local text

	seconds = math.floor(arg1)
	deciseconds = math.floor((10 * (arg1 - seconds)) + 0.5)
	if deciseconds == 10 then
		seconds = seconds + 1
		deciseconds = 0
	end
	hours = math.floor(seconds/3600)
	seconds = seconds - hours*3600
	minutes = math.floor(seconds/60)
	seconds = seconds - minutes*60

	return string.format("%02d:",minutes) .. string.format("%02d",seconds) .. string.format(".%01d",deciseconds)
end

function Recap_SetClassIcon(id,class)

	if class and recap_temp.ClassIcons[class] then
		getglobal("RecapList"..id.."_Class"):SetTexCoord(recap_temp.ClassIcons[class].left,recap_temp.ClassIcons[class].right,recap_temp.ClassIcons[class].top,recap_temp.ClassIcons[class].bottom)
	else
		getglobal("RecapList"..id.."_Class"):SetTexCoord(.9,1,.9,1)
	end
end

function Recap_SetFactionIcon(id,faction)

	item = getglobal("RecapList"..id.."_Faction")

	if faction and recap_temp.FactionIcons[faction] then
		item:SetTexture(recap_temp.FactionIcons[faction])
	else
		item:SetTexture("")
	end

end

function Recap_GetUnitInfo(unit)

	local u=(UnitName(unit))

	if u then

		if recap.Combatant[u] then
			_,recap.Combatant[u].Class = UnitClass(unit)
			_,recap.Combatant[u].Faction = UnitFactionGroup(unit)
			if UnitCreatureFamily(unit) and recap.Combatant[u].Faction then
				recap.Combatant[u].Class = "Pet"
			end
			recap.Combatant[u].Level = UnitLevel(unit)
		end

		if unit~="target" and unit~="mouseover" then

			-- Hawksy: create the 'recap_temp.Last' table for this unit so that we
			--   can have HP information from the beginning of a fight, in order to
			--   be able to calculate more accurate overhealing information.  The
			--   code will add party members or raid members, who would normally
			--   soon become combatants anyway.  We're just doing it a bit early.
			--   See also the call to Recap_MakeFriends in Recap_StartFight.
			if not recap.Combatant[u] then
				Recap_CreateBlankCombatant(u)
			end
			if not recap_temp.Last[u] then
				recap_temp.Last[u] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, Heal = 0 }
			end

			if recap_temp.Last[u] then
				if UnitIsVisible(unit) then
					recap_temp.Last[u].HP = UnitHealth(unit)
					recap_temp.Last[u].MaxHP = UnitHealthMax(unit)
				else
					recap_temp.Last[u].HP = nil -- discredit
				end
			end
		end
	end
end

-- creates a new .Combatant[arg1] that is a copy of DefaultCombatant
function Recap_CreateBlankCombatant(thisCombatant)

	if not recap.Combatant[thisCombatant] then

		-- Hawksy: modified following to be correct and complete (to reduce 'nil' errors)
		recap.Combatant[thisCombatant] = {
			TotalTime = 0,
			TotalTimeIn = 0,
			TotalTimeHeal = 0,
			TotalDmgIn = 0,
			TotalDmgOut = 0,
			TotalDPS = 0,
			TotalDPSIn = 0,
			TotalHPS = 0,
			TotalMaxHit = 0,
			TotalKills = 0,
			TotalHeal = 0,
			LastTime = 0,
			LastTimeIn = 0,
			LastTimeHeal = 0,
			LastDmgIn = 0,
			LastDmgOut = 0,
			LastDPS = 0,
			LastDPSIn = 0,
			LastHPS = 0,
			LastMaxHit = 0,
			LastKills = 0,
			LastHeal = 0,
			WasInLast = false,
			WasInCurrent = false,
			Friend = false,
			Ignore = nil,
			OwnedBy = nil,
			OwnsPet = nil,
			Name = nil,
			Class = nil,
			Faction = nil,
			Level = nil,
			Incoming = {},
			Detail = {}
		}
	end
end

-- populates .Opt with a copy of DefaultOpt
function Recap_LoadDefaultOpt()

	local i, j

	recap.Opt = {}
	for i in pairs(recap_temp.DefaultOpt) do
		recap.Opt[i] = {}
		for j in pairs(recap_temp.DefaultOpt[i]) do
			recap.Opt[i][j] = recap_temp.DefaultOpt[i][j]
		end
	end
end

function Recap_SaveOpt(user)

	local i, j

	if user and string.len(user)>0 then
		recap[user].Opt = {}
		for i in pairs(recap.Opt) do
			text = ""
			for j in pairs(recap.Opt[i]) do
				text = text..tostring(j).."="..tostring(recap.Opt[i][j]).." "
			end
			recap[user].Opt[i] = text
		end
	end
end

function Recap_LoadOpt(user)

	local i, j

	if user and recap[user] and recap[user].Opt then
		for i in pairs(recap[user].Opt) do
			if recap_temp.DefaultOpt[i] then
				for t,v in string.gmatch(recap[user].Opt[i],"(%w+)=(%d-%--%w+) ") do
					if v=="true" then
						v = true
					elseif v=="false" then
						v = false
					elseif tonumber(v) then
						v = tonumber(v)
					end
					if t=="type" and recap_temp.DefaultOpt[i][t] then
						v = recap_temp.DefaultOpt[i][t]
					end
					if recap.Opt[i] then
						recap.Opt[i][t] = v
					end
				end
			end
		end

		-- Hawksy: populate current user with any Default options that are missing,
		--   to prevent many 'nil' errors result from missing options.
		for i in pairs(recap_temp.DefaultOpt) do
			if not recap[user].Opt[i] then
				recap[user].Opt[i] = {}
				for j in pairs(recap_temp.DefaultOpt[i]) do
					recap[user].Opt[i][j] = recap_temp.DefaultOpt[i][j]
				end
			end
		end
	end
end

-- saves current data to data set "filen", overwrite~=nil to overwrite existing data
function Recap_SaveCombatants(filen,overwrite)

	local i, j, k
	local sd = {}

	if not filen or string.len(filen)<1 then
		return
	end

	if not recap_set[filen] then
		recap_set[filen] = {}
		recap_set[filen].TimeStamp = date()
		recap_set[filen].TotalDuration = 0
		recap_set[filen].Combatant = {}
	end

	recap_set[filen] = {}
	recap_set[filen].TimeStamp = date()
	recap_set[filen].TotalDuration = recap[recap_temp.p].TotalDuration
	recap_set[filen].Combatant = {}
	for i in pairs(recap.Combatant) do
		if (not recap.Opt.SaveFriends.value or recap.Combatant[i].Friend) then

			-- Hawksy: upgrade to 3.54, new values TotalTimeIn and TotalTimeHeal
			if not recap.Combatant[i].TotalTimeIn then
				recap.Combatant[i].TotalTimeIn = 0
			end
			if not recap.Combatant[i].TotalTimeHeal then
				recap.Combatant[i].TotalTimeHeal = 0
			end

			recap_set[filen].Combatant[i] = string.format("%s %d %d %d %d %d %.3f %.3f %.3f ~%d %d %d",
											tostring(recap.Combatant[i].Friend),
											recap.Combatant[i].TotalDmgIn,
											recap.Combatant[i].TotalDmgOut,
											recap.Combatant[i].TotalMaxHit,
											recap.Combatant[i].TotalHeal,
											recap.Combatant[i].TotalKills,
											recap.Combatant[i].TotalTime,
											recap.Combatant[i].TotalTimeIn,
											recap.Combatant[i].TotalTimeHeal,
											Recap_MakeKey(recap.Combatant[i].Faction),
											Recap_MakeKey(recap.Combatant[i].Class),
											recap.Combatant[i].Level or 0)
			if recap.Opt.LightData and not recap.Opt.LightData.value then
				-- save .Incoming
				recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."^"
				for k in pairs(recap.Combatant[i].Incoming) do
					if recap.Combatant[i].Incoming[k]>0 then
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..inckey[k]..recap.Combatant[i].Incoming[k]
					end
				end
				recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."^"

				for j in pairs(recap.Combatant[i].Detail) do
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."["..j..">"
					for k in pairs(recap.Combatant[i].Detail[j]) do
						recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i]..detkey[k]..recap.Combatant[i].Detail[j][k]
					end
					recap_set[filen].Combatant[i] = recap_set[filen].Combatant[i].."]"
				end

			end
		end
	end

	fight_count = 0
	for i in pairs(recap_set[filen].Combatant) do
		fight_count = fight_count + 1
	end
	recap_set[filen].ListSize = fight_count
end

function Recap_MakeKey(arg1)

	local i
	local key = 0

	for i in pairs(recap_temp.Keys) do
		if arg1==recap_temp.Keys[i] then
			key = i
		end
	end
	return key
end

function Recap_GetKey(arg1)

	local key = nil

	if not arg1 or not tonumber(arg1) or arg1==0 then
		key=nil
	else
		key=recap_temp.Keys[tonumber(arg1)]
	end
	return key
end

-- Hawksy: if Self Data Mode, check if the combatant is the player or the player's pet
function Recap_CheckSelfMode(combatant, ownedBy)

	local okayToLoad = false

	if recap.Opt.SelfData.value then
		-- Self Data Mode, is this the player or player's pet?
		if combatant == recap_temp.Player then
			-- player
			okayToLoad = true
		elseif ownedBy and (ownedBy == recap_temp.Player) then
			-- player's pet
			okayToLoad = true
		end
	else
		okayToLoad = true
	end

	return okayToLoad
end

function Recap_LoadCombatants(filen,overwrite)

	local i, j, k
	local sd = {} -- temp holding place for fields that may not exist
	local found, set_friend, set_dmgin, set_dmgout, set_maxhit, set_heal, set_kills, set_timeOut, set_timeIn, set_timeHeal, set_faction, set_class, set_level
	local okayToLoad

	if not filen or string.len(filen)<1 or not recap_set[filen] then
		return
	end

	Recap_ResetAllCombatants(false)

	recap[recap_temp.p].TotalDuration = recap[recap_temp.p].TotalDuration + recap_set[filen].TotalDuration

	for i in pairs(recap_set[filen].Combatant) do

		-- Hawksy: check for Self Data Mode, and if so only load the player (and pets if any)
		okayToLoad = Recap_CheckSelfMode(i, recap_set[filen].Combatant[i].OwnedBy)
		if okayToLoad then
			if not recap.Combatant[i] then
				Recap_CreateBlankCombatant(i)
			end

			-- Hawksy: format from 3.54 and later
			found,_,set_friend,set_dmgin,set_dmgout,set_maxhit,set_heal,set_kills,set_timeOut,set_timeIn,set_timeHeal = string.find(recap_set[filen].Combatant[i],"(%w+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+%.?%d+) (%d+%.?%d+) (%d+%.?%d+)")
			if found then
				if set_friend == "true" then
					recap.Combatant[i].Friend = true
				end
				if tonumber(set_maxhit) > recap.Combatant[i].TotalMaxHit then
					recap.Combatant[i].TotalMaxHit = tonumber(set_maxhit)
				end
				recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn + tonumber(set_dmgin)
				recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut + tonumber(set_dmgout)
				recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills + tonumber(set_kills)
				recap.Combatant[i].TotalHeal = recap.Combatant[i].TotalHeal + tonumber(set_heal)
				recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime + tonumber(set_timeOut)
				if recap.Combatant[i].TotalTime > recap_temp.MinTime then
					recap.Combatant[i].TotalDPS = recap.Combatant[i].TotalDmgOut/recap.Combatant[i].TotalTime
				else
					recap.Combatant[i].TotalDPS = 0
				end
				recap.Combatant[i].TotalTimeIn = recap.Combatant[i].TotalTimeIn + tonumber(set_timeIn)
				if recap.Combatant[i].TotalTimeIn > recap_temp.MinTime then
					recap.Combatant[i].TotalDPSIn = recap.Combatant[i].TotalDmgIn/recap.Combatant[i].TotalTimeIn
				else
					recap.Combatant[i].TotalDPSIn = 0
				end
				recap.Combatant[i].TotalTimeHeal = recap.Combatant[i].TotalTimeHeal + tonumber(set_timeHeal)
				if recap.Combatant[i].TotalTimeHeal > recap_temp.MinTime then
					recap.Combatant[i].TotalHPS = recap.Combatant[i].TotalHeal/recap.Combatant[i].TotalTimeHeal
				else
					recap.Combatant[i].TotalHPS = 0
				end
			else
				-- Hawksy: load format from 3.53 and earlier
				found,_,set_friend,set_dmgin,set_dmgout,set_maxhit,set_heal,set_kills,set_timeOut = string.find(recap_set[filen].Combatant[i],"(%w+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+%.?%d+)")
				if found then
					if set_friend == "true" then
						recap.Combatant[i].Friend = true
					end
					if tonumber(set_maxhit) > recap.Combatant[i].TotalMaxHit then
						recap.Combatant[i].TotalMaxHit = tonumber(set_maxhit)
					end
					recap.Combatant[i].TotalDmgIn = recap.Combatant[i].TotalDmgIn + tonumber(set_dmgin)
					recap.Combatant[i].TotalDmgOut = recap.Combatant[i].TotalDmgOut + tonumber(set_dmgout)
					recap.Combatant[i].TotalKills = recap.Combatant[i].TotalKills + tonumber(set_kills)
					recap.Combatant[i].TotalHeal = recap.Combatant[i].TotalHeal + tonumber(set_heal)
					recap.Combatant[i].TotalTime = recap.Combatant[i].TotalTime + tonumber(set_timeOut)
					if recap.Combatant[i].TotalTime > recap_temp.MinTime then
						recap.Combatant[i].TotalDPS = recap.Combatant[i].TotalDmgOut/recap.Combatant[i].TotalTime
					else
						recap.Combatant[i].TotalDPS = 0
					end
					recap.Combatant[i].TotalTimeIn = 0
					recap.Combatant[i].TotalDPSIn = 0
					recap.Combatant[i].TotalTimeHeal = 0
					recap.Combatant[i].TotalHPS = 0
				end
			end
			found,_,set_faction,set_class,set_level = string.find(recap_set[filen].Combatant[i],"~(%d+) (%d+) (%d+)")
			if not found then
				found,_,set_faction,set_class = string.find(recap_set[filen].Combatant[i],"~(%d+) (%d+)")
			end
			if found then
				recap.Combatant[i].Faction = Recap_GetKey(set_faction)
				recap.Combatant[i].Class = Recap_GetKey(set_class)
				recap.Combatant[i].Level = recap.Combatant[i].Level or set_level or 0
			end

			if not recap.Opt.LightData.value then

				-- process incoming: ^a000b000c000^
				for k in pairs(sd) do sd[k] = nil end -- wipe temp buffer
				found,_,j = string.find(recap_set[filen].Combatant[i],"%^(.-)%^")
				if found then
					for k in pairs(inckey) do
						_,_,sd[k] = string.find(j,inckey[k].."(%d+)")
					end
				elseif string.find(recap_set[filen].Combatant[i],"%^%d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+") then
					-- Hawksy: format from version 3.53 onwards to accommodate NonMeleeTicks
					found,_,sd.MeleeDamage,sd.MeleeMax,sd.MeleeHits,sd.MeleeCrits,sd.MeleeMissed,sd.MeleeDodged,sd.MeleeParried,sd.MeleeBlocked,
						sd.NonMeleeDamage,sd.NonMeleeMax,sd.NonMeleeHits,sd.NonMeleeCrits,sd.NonMeleeMissed,sd.NonMeleeTicks =
							string.find(recap_set[filen].Combatant[i],"%^(%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+)")
				elseif string.find(recap_set[filen].Combatant[i],"%^%d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+ %d+") then
					-- Hawksy: format from version 3.52 and earlier
					found,_,sd.MeleeDamage,sd.MeleeMax,sd.MeleeHits,sd.MeleeCrits,sd.MeleeMissed,sd.MeleeDodged,sd.MeleeParried,sd.MeleeBlocked,
						sd.NonMeleeDamage,sd.NonMeleeMax,sd.NonMeleeHits,sd.NonMeleeCrits,sd.NonMeleeMissed =
							string.find(recap_set[filen].Combatant[i],"%^(%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+)")
				end
				if found then
					for k in pairs(sd) do
						if sd[k] and tonumber(sd[k])>0 then
							recap.Combatant[i].Incoming[k] = tonumber(sd[k])
						end
					end
				end

				-- process details: [name:a000b000c000]
				for j in string.gmatch(recap_set[filen].Combatant[i],"%[.-%]") do
					for k in pairs(sd) do sd[k] = nil end -- wipe temp buffer
					found,_,set_name = string.find(j,"%[(.-)%>")
					if found then -- new saved method
						recap.Combatant[i].Detail[set_name] = {}
						for k in pairs(detkey) do
							_,_,sd[k] = string.find(j,detkey[k].."(%d+)")
						end
					else
						found,_,set_name = string.find(j,"%[(.-) %d")
						if found then -- old saved method
							recap.Combatant[i].Detail[set_name] = {}
							_,_,sd.HitsDmg,sd.Hits,sd.HitsMax,sd.CritsDmg,sd.Crits,sd.CritsMax,sd.CritsEvents,sd.Missed =
								string.find(j,"%[.+ (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+)") -- old method
						end
					end
					if found then
						for k in pairs(sd) do
							if sd[k] and tonumber(sd[k])>0 then
								recap.Combatant[i].Detail[set_name][k] = tonumber(sd[k])
							end
						end
					end
				end
			end
		end
	end
end

function Recap_ResetAllCombatants(keepPlayer)

	local current_state = recap.Opt.State.value
	recap.Opt.State.value = "Stopped"

	if keepPlayer then
		-- scan combatants and save a copy of the player (and player's pets, if any)
		local i
		local local_save = {}
		for i in pairs(recap.Combatant) do
			if Recap_CheckSelfMode(i, (recap.Combatant[i] and recap.Combatant[i].OwnedBy)) then
				local_save[i] = recap.Combatant[i]
			end
		end
		-- clear all combatants and then restore the saved combatants
		recap.Combatant = {}
		for i in pairs(local_save) do
			recap.Combatant[i] = local_save[i]
		end
	else
		recap.Combatant = {}
	end
	recap_temp.IdleTimer = -1
	recap_temp.Last = {}
	recap[recap_temp.p].LastDuration = 0
	recap[recap_temp.p].TotalDuration = 0
	recap_temp.FightStart = 0
	recap_temp.FightEnd = 0
	recap_temp.ListSize = 0
	recap_temp.List = {}

	-- Hawksy: if Recent Data Mode is active, clear the buffers
	if recap.Opt.RecentData.value then
		Recap_CreateRecent()
	else
		recap_temp.Recent = {}
	end

	recap.Opt.State.value = current_state
end

local function Recap_LoadSelf()

	recap.Self = recap.Self or {}
	recap.Self[recap_temp.s] = recap.Self[recap_temp.s] or {}

	recap_temp.SelfList = {}
	recap_temp.SelfListSize = 1
end

function Recap_InitializeData()

	local i, j, k, l, m
	local temp_user = recap_temp.p
	local oldkeys = { ["Krieger"]="WARRIOR", ["Magier"]="MAGE", ["Schurke"]="ROGUE", ["Druide"]="DRUID",
					  ["J\195\164ger"]="HUNTER", ["Schamane"]="SHAMAN", ["Priester"]="PRIEST", ["Hexenmeister"]="WARLOCK",
					  ["Guerrier"]="WARRIOR", ["Voleur"]="ROGUE", ["Chasseur"]="HUNTER", ["Chaman"]="SHAMAN",
					  ["Prêtre"]="PRIEST", ["Démoniste"]="WARLOCK", ["Familier"]="Pet" }

	if not recap.DataVersion then
		-- wipe all pre-2.6 data
		recap = {}
		recap.DataVersion = 3.2
	elseif recap.DataVersion < 3.0 then
		-- convert 2.6 to 3.0, make classes uppercase
		for i in pairs(recap.Combatant) do

			if recap.Combatant[i].Faction=="Allianz" then
				recap.Combatant[i].Faction = "Alliance"
			end

			if recap.Combatant[i].Class then
				-- convert classes to english, which all locales will use for icon reference
				if oldkeys[recap.Combatant[i].Class] then
					recap.Combatant[i].Class = oldkeys[recap.Combatant[i].Class]
				end
				recap.Combatant[i].Class = string.upper(recap.Combatant[i].Class)
			end
		end
		recap.DataVersion = 3.0
	end

	-- If no Opt, create a default one
	if not recap.Opt then
		Recap_LoadDefaultOpt()
	end

	-- if this user hasn't been made before
	if not recap[recap_temp.p] then
		recap[recap_temp.p] = {}
		recap[recap_temp.p].TotalDuration = 0
		recap[recap_temp.p].LastDuration = 0
	end

	-- if this is a different/first-time user, or if this is first run of a new version,
	-- save the current combatants and options and then load them based on default.
	-- any changes in combatant or option format must come with a version change
	if (recap.User ~= recap_temp.p) or recap.DataVersion<Recap_Version then

		-- if we have data from previous user, save them
		if recap.User then
			recap_temp.p = recap.User
			Recap_SaveCombatants("UserData:"..recap.User,1)
			Recap_SaveOpt(recap.User)
			recap_temp.p = temp_user
		end
		recap.User = recap_temp.p

		-- wipe slate, load defaults and overlay saved data ontop
		Recap_LoadDefaultOpt()
		Recap_ResetAllCombatants(false)

		-- convert data sets to new format (see readme)
		if recap.DataVersion < 3.2 then
			for i in pairs(recap_set) do
				if recap_set[i].Combatant then
					j = recap_set[i].TimeStamp
					Recap_LoadCombatants(i,1)
					for k in pairs(recap_set[i].Combatant) do
						l = (l or 0) + string.len(recap_set[i].Combatant[k])
					end
					Recap_SaveCombatants(i,1)
					for k in pairs(recap_set[i].Combatant) do
						m = (m or 0) + string.len(recap_set[i].Combatant[k])
					end
					recap_set[i].TimeStamp = j
				end
			end
			recap.DataVersion = 3.2
		end

		Recap_LoadCombatants("UserData:"..recap_temp.p,1)
		Recap_LoadOpt(recap_temp.p)

		recap.DataVersion = Recap_Version

	else
		for i in pairs(recap.Combatant) do
			recap_temp.Last[i] = { StartOut = 0, EndOut = 0, StartIn = 0, EndIn = 0, StartHeal = 0, EndHeal = 0, DmgIn = 0, DmgOut = 0, MaxHit = 0, Kills = 0, Heal = 0 }
		end
	end

	-- remove current UserData: - it's now "live" in recap.Combatant
	if recap_set and recap_set["UserData:"..recap_temp.p] then
		recap_set["UserData:"..recap_temp.p] = nil
	end

	Recap_LoadSelf()

	-- Hawksy: special case correction for some dual damage/healing tickers
	Recap_CorrectDualDoTHoT()

end

--[[ Context menu functions ]]

function RecapMenu_OnUpdate(arg1)

	local i

	if not recap_temp.MenuTimer or MouseIsOver(RecapMenu) then
		recap_temp.MenuTimer = 0
	end

	recap_temp.MenuTimer = recap_temp.MenuTimer + arg1
	-- Hawksy: doubled the time that a menu hangs around
	if recap_temp.MenuTimer > 0.5 then
		recap_temp.MenuTimer = 0
		RecapMenu:Hide()
		if recap_temp.SelectedEffect then
			for i=1,15 do
				getglobal("RecapSelf"..i):UnlockHighlight()
			end
		end
	end
end


--[[ Fight report functions ]]

function Recap_AutoPostGetStatID(arg1)

	if arg1=="Damage" then
		return "DmgOut"
	elseif arg1=="Tanking" then
		return "DmgIn"
	elseif arg1=="Healing" then
		return "Heal"
	end
	return "DPS"
end

function Recap_PostFight()

	local print_chat = SendChatMessage
	local chatchan, chatnum, text

	if recap.Opt.SpamRows.value then
		chatchan = "recap auto"
		chatnum = nil
		Recap_PostSpamRows(Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat), chatchan, chatnum)
	else
		chatchan = recap.Opt.AutoPost.Channel
		if chatchan == nil then
			chatchan = "Self"
		end
		_,_,chatnum = string.find(chatchan,"(%d+)")
		if chatnum then
			chatchan = "CHANNEL"
		end
		if chatchan=="Self" then
			print_chat = Recap_Print
		end
		text = Recap_PostSpamLine(Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat))
		if text and string.find(text,"%d") then
			print_chat(Recap_PostSpamLine(Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat)),chatchan,nil,chatnum)
		end
	end

end

function Recap_PostLeader()

	local print_chat = SendChatMessage
	local i, chatchan, chatnum, leader, statid, thisCombatant

	if recap_temp.ListSize>2 then
		chatchan = recap.Opt.AutoPost.Channel
		if chatchan == nil then
			chatchan = "Self"
		end
		_,_,chatnum = string.find(chatchan,"(%d+)")
		if chatnum then
			chatchan = "CHANNEL"
		end
		if chatchan=="Self" then
			print_chat = Recap_Print
		end
		statid = Recap_AutoPostGetStatID(recap.Opt.AutoPost.Stat)

		if not recap.Combatant[recap_temp.Leader] then -- in case leader reset
			recap_temp.Leader = nil
		end

		-- Hawksy: guard code
		thisCombatant = recap_temp.List[1].Name
		if (not recap_temp.Leader) and thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
			recap_temp.Leader = thisCombatant
			leader = recap_temp.Leader -- spam initial leader
		end

		if recap_temp.Leader then
			for i in pairs(recap.Combatant) do
				if (recap.Combatant[i]["Total"..statid] > recap.Combatant[recap_temp.Leader]["Total"..statid]) and recap.Combatant[i].Friend then
					leader = i
				end
			end
			if leader then
				recap_temp.Leader = leader
				print_chat(string.format("New leader for %s: %s with %s",recap.Opt.AutoPost.Stat,leader,Recap_FormatStat(statid,leader)),chatchan,nil,chatnum)
			end
		end
	end
end


-- returns 'text' to be spammed in autopost or linked in chat
function Recap_PostSpamLine(arg1)

	local text, i

	text = recap_temp.Local.LinkRank[arg1].." for "..recap_temp.Local.LastAll[recap.Opt.View.value]..":"
	for i=1,recap.Opt.MaxRank.value do
		-- Hawksy: guard code
		if recap_temp.List[i] then
			thisCombatant = recap_temp.List[i].Name
			if (i<recap_temp.ListSize) and thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				if recap_temp.List[i][arg1]>0 then
					text = text.." ["..i.."."..thisCombatant.." "..Recap_FormatStat(arg1,i).."]"
				end
			end
		end
	end
	return text
end

-- arg1 = stat to report by (DPS, DmgOut, etc)
function Recap_PostSpamRows(arg1, chatchan, chatnum)

	local i
	local headertext = nil
	local print_chat = SendChatMessage

	arg1 = string.gsub(arg1,"P$","") -- strip trailing P from % id's

	if chatchan=="recap auto" then
		chatchan = recap.Opt.AutoPost.Channel
	end
	if not chatchan then
		chatchan = "Self"
	end
	if not chatnum then
		_,_,chatnum = string.find(chatchan,"(%d+)")
		if chatnum then
			chatchan = "CHANNEL"
		end
	end
	if chatchan=="Self" then
		print_chat = Recap_Print
	end
	headertext = "__ "..recap_temp.Local.LinkRank[arg1].." for "..recap_temp.Local.LastAll[recap.Opt.View.value].." __"
	for i=1,recap.Opt.MaxRank.value do
		-- Hawksy: guard code
		if recap_temp.List[i] and recap_temp.List[i].Name then
			thisCombatant = recap_temp.List[i].Name
			if (i<recap_temp.ListSize) and thisCombatant and recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].Friend then
				text = i..". "..thisCombatant..": "..Recap_FormatStat(arg1,i)
				if recap_temp.List[i][arg1] > 0 then
					if headertext then
						print_chat(headertext,chatchan,nil,chatnum)
						headertext = nil
					end
					print_chat(text,chatchan,nil,chatnum)
				end
			end
		end
	end
end

-- Hawksy: two functions added to report Recent Events
function Recap_PostRecentRows(outgoing, filterCombatant, filterEffect, chatchan)

	local i, last, index, message, sentEOF
	local print_chat = SendChatMessage
	local chatnum = nil -- if channel is "Channel", this is the channel numbers

	-- block posting to channels that would be excessively annoying to others
	--   (also blocks other channels that happen to have BlockChannels as substrings)
	if chatchan then
		for i=1,#recap_temp.BlockChannels do
			if string.find(string.lower(chatchan), recap_temp.BlockChannels[i]) then
				PlaySound("igQuestFailed")
				return
			end
		end
	end

	if chatchan=="WHISPER" then
		chatnum = ChatFrameEditBox:GetAttribute("tellTarget")
	elseif chatchan=="CHANNEL" then
		chatnum = ChatFrameEditBox:GetAttribute("channelTarget")
	end
	if not chatnum then
		_,_,chatnum = string.find(chatchan,"(%d+)")
		if chatnum then
			chatchan = "CHANNEL"
		end
	end
	if chatchan=="Self" then
		print_chat = Recap_Print
	end

	if recap_temp.Recent then
		i = recap_temp.Recent.First
		last = recap_temp.Recent.Last
		print_chat("----  vvvv  ----",chatchan,nil,chatnum)
		if (i == last) and not (recap_temp.Recent[i] and recap_temp.Recent[i].T) then
			-- no data at all
		else
			-- data exists, post the rows (there may be none due to filtering)
			sentEOF = true -- set to skip any initial 'end of fight' markers
			for index=1, recap.Opt.RecentEventCount.value do
				if recap_temp.Recent[i].S == "**EOF**" then
					-- maybe post an 'end of fight' marker
					if sentEOF == true then
						-- avoid sending two in a row, and avoid leading 'end of fight' indicators
					else
						-- special 'end of fight' message
						message = RECAP_RECAP.." "..Recap_FormatTimeRecent(recap_temp.Recent[i].T).." --------------------------------------------------"
						print_chat(message, chatchan, nil, chatnum)
						sentEOF = true
					end
				else
					-- not an 'end of fight'
					message = Recap_FormatRecentEvent(i, outgoing, filterCombatant, filterEffect)
					-- if filterCombatant or filterEffect fail, the message gets returned as nil
					if message then
						print_chat(message, chatchan, nil, chatnum)
						sentEOF = false
					end
				end
				if (i == last) then
					-- just posted the most recent one, leave the loop
					break
				end
				-- increment circular pointer
				if i >= recap.Opt.RecentEventCount.value then
					i = 1
				else
					i = i + 1
				end
			end
		end
		print_chat("----  ^^^^  ----",chatchan,nil,chatnum)
	end
end

function Recap_FormatRecentEvent(i, outgoing, filterCombatant, filterEffect)

	local printTime = Recap_FormatTimeRecent(recap_temp.Recent[i].T)
	local source = recap_temp.Recent[i].S
	local dest = recap_temp.Recent[i].D
	local effectType = string.sub(recap_temp.Recent[i].E,1,1)
	local printEffect = string.sub(recap_temp.Recent[i].E,2)
	local amountNumeric = false
	if type(recap_temp.Recent[i].A) == "number" then
		amountNumeric = true
	end
	local printAmount = tostring(recap_temp.Recent[i].A)
	local printCrit
	if recap_temp.Recent[i].C then
		printCrit = "**"
	else
		printCrit = ""
	end

	local okayCombatant = false
	if filterCombatant then
		-- filtering by combatant
		if recap.Opt.MergePets.value and recap.Combatant[filterCombatant].OwnsPet then
			-- merge is active and combatant has pets, see if this source or destination is one of their pets
			if outgoing and recap.Combatant[source] and (recap.Combatant[source].OwnedBy == filterCombatant) then
				okayCombatant = true
			elseif not outgoing and recap.Combatant[dest] and (recap.Combatant[dest].OwnedBy == filterCombatant) then
				okayCombatant = true
			end
		end
		-- check the combatant
		if outgoing and (filterCombatant == source) then
			okayCombatant = true
		elseif not outgoing and (filterCombatant == dest) then
			okayCombatant = true
		end
	else
		-- no filtering by combatant
		okayCombatant = true
	end
	if okayCombatant == false then
		return nil
	end

	-- filter by effect
	if filterEffect and (filterEffect ~= printEffect) then
		return nil
	end

	-- prepare the message
	if amountNumeric then
		if effectType == "1" then
			-- damage
			return RECAP_RECAP.." "..printTime.." "..source.." ("..printEffect..") => "..dest.." ( -"..printAmount..printCrit.." )"
		else
			-- heal
			return RECAP_RECAP.." "..printTime.." "..source.." ("..printEffect..") => "..dest.." ( +"..printAmount..printCrit.." )"
		end
	else
		return RECAP_RECAP.." "..printTime.." "..source.." ("..printEffect..") => "..dest.." ( "..printAmount.." )"
	end
end

-- returns a formatted string for arg1 stat ("DPS" "Time" etc) for combatant arg2 (number is .List[], non-number is name)
function Recap_FormatStat(arg1,arg2)

	local i

	arg1 = string.gsub(arg1,"P$","")

	if not tonumber(arg2) then

		for i=1,(recap_temp.ListSize-1) do
			if recap_temp.List[i].Name==arg2 then
				arg2 = i
				i = recap_temp.ListSize
			end
		end
	end

	if tonumber(arg2) then

		if arg1=="DPS" then
			return string.format("%.1f",recap_temp.List[arg2].DPS)
		elseif arg1=="DPSIn" then
			return string.format("%.1f",recap_temp.List[arg2].DPSIn)
		elseif arg1=="HPS" then
			return string.format("%.1f",recap_temp.List[arg2].HPS)
		elseif arg1=="DPSvsAll" then
			return string.format("%.1f",recap_temp.List[arg2].DPSvsAll)
		elseif arg1=="Time" then
			return Recap_FormatTime(recap_temp.List[arg2].Time)
		elseif arg1=="DmgOut" then
			return recap_temp.List[arg2].DmgOut.." ("..recap_temp.List[arg2].DmgOutP.."%)"
		elseif arg1=="DmgIn" then
			return recap_temp.List[arg2].DmgIn.." ("..recap_temp.List[arg2].DmgInP.."%)"
		elseif arg1=="Heal" then
			return recap_temp.List[arg2].Heal.." ("..recap_temp.List[arg2].HealP.."%)".." ("..recap_temp.List[arg2].Over.."%>)"
		elseif arg1=="Over" then
			return recap_temp.List[arg2].Over.."%"
		else
			return recap_temp.List[arg2][arg1]
		end
	end

	return ""
end


function Recap_Print(arg1)
	DEFAULT_CHAT_FRAME:AddMessage(tostring(arg1))
end

--[[ Debug ]]

function Recap_MoveMinimize()

	RecapCloseButton:ClearAllPoints()
	RecapCloseButton:SetPoint("TOPRIGHT", "RecapFrame", "TOPRIGHT", -6, -6)
	RecapMinStatus:ClearAllPoints()
	RecapMinStatus:SetPoint("TOPLEFT", "RecapFrame", "TOPLEFT", 6, -7)
	RecapResetButton:ClearAllPoints()
	RecapResetButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 5, 4)
	RecapMinimizeButton:ClearAllPoints()

	if recap.Opt.GrowLeftwards.value and not recap.Opt.GrowUpwards.value then -- topright
		RecapMinimizeButton:SetPoint("TOPRIGHT", "RecapFrame", "TOPRIGHT", -6, -6)
		RecapCloseButton:SetPoint("TOPRIGHT", "RecapMinimizeButton", "TOPLEFT", -2, 0)
	elseif recap.Opt.GrowLeftwards.value and recap.Opt.GrowUpwards.value then -- bottomright
		RecapMinimizeButton:SetPoint("BOTTOMRIGHT", "RecapFrame", "BOTTOMRIGHT", -6, 6)
		if recap.Opt.Minimized.value then
			RecapCloseButton:SetPoint("TOPRIGHT", "RecapMinimizeButton", "TOPLEFT", -2, 0)
		end
	elseif not recap.Opt.GrowLeftwards.value and not recap.Opt.GrowUpwards.value then -- topleft
		RecapMinimizeButton:SetPoint("TOPLEFT", "RecapFrame", "TOPLEFT", 6, -6)
		RecapMinStatus:SetPoint("TOPLEFT", "RecapMinimizeButton", "TOPRIGHT", 2, -1)
	else -- bottomleft
		RecapMinimizeButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 6, 6)
		RecapResetButton:SetPoint("BOTTOMLEFT", "RecapFrame", "BOTTOMLEFT", 23, 4)
	end

	if recap.Opt.Minimized.value and not recap.Opt.GrowLeftwards.value then
		RecapMinStatus:SetPoint("TOPLEFT", "RecapMinimizeButton", "TOPRIGHT", 2, -1)
	end

	if recap.Opt.AutoMinimize.value then
		RecapMinimizeButton:SetWidth(1)
		RecapMinimizeButton:Hide()
	else
		RecapMinimizeButton:SetWidth(16)
		RecapMinimizeButton:Show()
	end

end

function RecapMenu_SetCheck(id,arg1)

	if arg1 then
		getglobal("RecapMenu"..id.."_Check"):Show()
	else
		getglobal("RecapMenu"..id.."_Check"):Hide()
	end
end

function RecapMenu_OnClick()

	local i
	local id, thisCombatant = this:GetID()

	if recap_temp.Menu==recap_temp.ColumnMenu or recap_temp.Menu==recap_temp.MinMenu or recap_temp.Menu==recap_temp.EffectOptMenu then
		thisCombatant = recap_temp.Menu[id].Info -- .Info of menu item selected
		if recap.Opt[thisCombatant] then
			recap.Opt[thisCombatant].value = not recap.Opt[thisCombatant].value
			getglobal("RecapOptCheck_"..recap_temp.Menu[id].Info):SetChecked(0)
			if recap.Opt[thisCombatant].value then
				getglobal("RecapOptCheck_"..recap_temp.Menu[id].Info):SetChecked(1)
			end
			RecapMenu_SetCheck(id,recap.Opt[thisCombatant].value)
			if recap.Opt.Minimized.value then
				Recap_Minimize()
			else
				Recap_Maximize()
			end
		elseif thisCombatant=="Pin" then
			recap.Opt.Pinned.value = not recap.Opt.Pinned.value
			Recap_SetButtons()
			RecapMenu:Hide()
		end
	end

	-- popup menu
	if recap_temp.Menu==recap_temp.AddMenu or recap_temp.Menu==recap_temp.DropMenu then
		RecapMenu:Hide()
		thisCombatant = recap_temp.List[recap_temp.Selected].Name -- name selected

		if id==1 and recap_temp.Menu==recap_temp.DropMenu and thisCombatant~=recap_temp.Player then

			-- Make them no longer a 'friendly combatant'
			recap.Combatant[thisCombatant].Friend = false

		elseif id==1 and thisCombatant~=recap_temp.Player then

			-- Toggle their 'friendly combatant' status
			recap.Combatant[thisCombatant].Friend = not recap.Combatant[thisCombatant].Friend

		elseif id==2 and recap.Opt.View.value=="All" then -- reset

			-- 'Reset' a combatant, meaning force all their numbers to zero
			recap.Combatant[thisCombatant] = nil
			Recap_CreateBlankCombatant(thisCombatant)

		elseif id==2 and recap.Opt.View.value=="Last" then

			-- Partially 'Reset' a combatant, meaning SUBTRACT their last fight information
			if recap.Combatant[thisCombatant].WasInLast then
				-- Hawksy: add a confirm dialog
				StaticPopupDialogs["RECAP_CONFIRMLASTRESET"] = {
					text = TEXT(recap_temp.Local.ConfirmLastReset),
					button1 = TEXT(ACCEPT),
					button2 = TEXT(CANCEL),
					OnAccept = function()
						-- SUBTRACT the last fight --
						Recap_ResetLastFight(thisCombatant)
						-- force redisplay (not sure if there might not be a cheaper call)
						Recap_Maximize()
					end,
					OnHide = function()
					end,
					OnCancel = function()
					end,
					timeout = 0,
					showAlert = 1,
					whileDead = 1
				}
				StaticPopup_Show("RECAP_CONFIRMLASTRESET")
			end

		elseif id==3 and thisCombatant~=recap_temp.Player then

			-- Put this combatant on 'ignore' (after forcing all their numbers to zero)
			recap.Combatant[thisCombatant] = nil
			Recap_CreateBlankCombatant(thisCombatant)
			recap.Combatant[thisCombatant].Ignore = true

		end

		Recap_SetView()
	else
		-- remove this effect
		if recap_temp.Menu==recap_temp.EffectMenu then
			RecapMenu:Hide()
			for i=1,15 do
				getglobal("RecapSelf"..i):UnlockHighlight()
			end
			if recap_temp.SelectedEffect and recap.Self[recap_temp.s][recap_temp.SelectedEffect] then
				recap.Self[recap_temp.s][recap_temp.SelectedEffect] = nil
				recap_temp.SelectedEffect = nil
				Recap_SetView()
			end
		end

		-- other actions
		if recap_temp.Menu==recap_temp.DetailMenu then
			if recap_temp.DetailsList and recap_temp.DetailsList[1][recap_temp.Menu[id].Info] then
				recap.Opt.PanelDetail.value = recap_temp.Menu[id].Info
				RecapPanel_Populate(recap_temp.List[recap_temp.Selected].Name)
			end
			RecapMenu:Hide()
		end
	end

end

-- offset=1 to move window so it's under pointer
function RecapCreateMenu(menu, offset)

	local i, cy, x, y, lines

	recap_temp.Menu = menu

	if recap_temp.Menu==recap_temp.ColumnMenu or recap_temp.Menu==recap_temp.MinMenu or recap_temp.Menu==recap_temp.EffectOptMenu then
		for i in pairs(recap_temp.Menu) do
			if recap.Opt[recap_temp.Menu[i].Info] then
				recap_temp.Menu[i].Check = recap.Opt[recap_temp.Menu[i].Info].value
			end
		end
	end

	if recap_temp.Menu==recap_temp.MinMenu then
		recap_temp.Menu[1].Check = recap.Opt.Pinned.value
	end

	if recap_temp.Menu==recap_temp.DetailMenu then
		for i in pairs(recap_temp.Menu) do
			recap_temp.Menu[i].Check = recap.Opt.PanelDetail.value==recap_temp.Menu[i].Info
		end
	end

	lines = min(#recap_temp.Menu,RECAP_MAX_MENU_LINES)

	cy = 44 -- 12+32
	for i=1,lines do
		getglobal("RecapMenu"..i.."_Text"):SetText(recap_temp.Menu[i].Text)
		RecapMenu_SetCheck(i,recap_temp.Menu[i].Check)
		getglobal("RecapMenu"..i):Show()
		cy = cy + 14
	end
	for i=(lines+1),RECAP_MAX_MENU_LINES do
		getglobal("RecapMenu"..i):Hide()
	end

	RecapMenu:SetHeight(cy)
	RecapDropSubFrame:SetHeight(cy-32)
	RecapMenu:ClearAllPoints()

	x,y = GetCursorPosition()
	if recap.Opt.GrowUpwards.value and not offset then
		y = this:GetBottom()*UIParent:GetScale()
		RecapMenu:SetPoint("BOTTOMRIGHT","UIParent","BOTTOMLEFT",(x/UIParent:GetScale())+20,(y/UIParent:GetScale())-4)
	else
		y = this:GetTop()*UIParent:GetScale()
		RecapMenu:SetPoint("TOPRIGHT","UIParent","BOTTOMLEFT",(x/UIParent:GetScale())+20,(y/UIParent:GetScale())+(offset and 16 or 4))
	end

	RecapMenu:Show()

end

function RecapMenu_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	id = this:GetID()
	if recap_temp.Menu[id] and recap_temp.Menu[id].Info then
		if recap_temp.Menu==recap_temp.AddMenu or recap_temp.Menu==recap_temp.DropMenu then
			Recap_OnTooltip(recap_temp.Menu[id].Info,recap_temp.List[recap_temp.Selected].Name)
		elseif recap_temp.Menu==recap_temp.EffectMenu and recap_temp.SelectedEffect then
			Recap_OnTooltip(recap_temp.Menu[id].Info,string.sub(recap_temp.SelectedEffect,2))
		else
			Recap_OnTooltip(recap_temp.Menu[id].Info)
		end
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapEffectsHeader_OnEnter()

	-- Hawksy: lock OnEnter/OnLeave events
	if recap_temp.OnEnterOnLeave == true then return end
	recap_temp.OnEnterOnLeave = true

	local s

	_,_,s = string.find(this:GetName(),"RecapHeader_(.+)")
	if s then
		Recap_OnTooltip("Header"..s)
	end

	-- Hawksy: unlock OnEnter/OnLeave events
	recap_temp.OnEnterOnLeave = false
end

function RecapSelf_OnMouseUp(arg1)

	recap_temp.SelectedEffect = nil
	Recap_OnMouseUp(arg1)
end

function RecapEffectsHeader_OnMouseUp(arg1)

	if arg1=="RightButton" then
		RecapCreateMenu(recap_temp.EffectOptMenu, nil)
	end
end

function RecapEffectsHeader_OnClick()

	local idx, i
	local alltext, text = ""
	local channel, chatnumber = ChatFrameEditBox:GetAttribute("chatType")
	local hasdata
	local arg1

	-- Hawksy: new code to support sorting effects on a per column basis
	if not IsShiftKeyDown() then
		arg1 = string.gsub(this:GetName(),"RecapHeader_","")
		if recap.Opt.ESortBy.value==string.gsub(arg1,"$","") then
			recap.Opt.ESortDir.value = not recap.Opt.ESortDir.value
		else
			recap.Opt.ESortBy.value = string.gsub(arg1,"$","")
			if arg1=="EName" then
				recap.Opt.ESortDir.value = true
			else
				recap.Opt.ESortDir.value = false
			end
		end
		Recap_SortSelfList()
		RecapScrollBar_Update()

	elseif IsShiftKeyDown() and ChatFrameEditBox:IsVisible() then

		if channel=="WHISPER" then
			chatnumber = ChatFrameEditBox:GetAttribute("tellTarget")
		elseif channel=="CHANNEL" then
			chatnumber = ChatFrameEditBox:GetAttribute("channelTarget")
		end

		_,_,idx = string.find(tostring(this:GetName()),"RecapHeader_(.+)")
		if idx and idx~="EName" then

			alltext = (recap.Opt.SpamRows.value and "__ " or "")..recap_temp.Player.."'s "..Recap_GetTooltip("Header"..idx)..(recap.Opt.SpamRows.value and " __" or ":")
			for i=1,recap_temp.SelfListSize-1 do
				if recap_temp.SelfList[i][idx]~="0.0%" and recap_temp.SelfList[i][idx]~="--" and tostring(recap_temp.SelfList[i][idx])~="0" then
					if not hasdata and recap.Opt.SpamRows.value then
						SendChatMessage(alltext,channel,nil,chatnumber)
					end
					hasdata = true
					text = string.sub(recap_temp.SelfList[i].EName,2)..(string.sub(recap_temp.SelfList[i].EName,1,1)=="3" and "*" or "")..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.SelfList[i][idx]..(idx=="ETotal" and (" ("..recap_temp.SelfList[i].ETotalP..")") or "")
					if not recap.Opt.SpamRows.value then
						alltext = alltext.." ["..text.."]"
					else
						SendChatMessage(text,channel,nil,chatnumber)
					end
				end
			end
			if not recap.Opt.SpamRows.value and hasdata then
				Recap_InsertChat(alltext)
			end
		end

	-- Hawksy: new post to "Self"
	elseif IsShiftKeyDown() then

		_,_,idx = string.find(tostring(this:GetName()),"RecapHeader_(.+)")
		if idx and idx~="EName" then

			alltext = (recap.Opt.SpamRows.value and "__ " or "")..recap_temp.Player.."'s "..Recap_GetTooltip("Header"..idx)..(recap.Opt.SpamRows.value and " __" or ":")
			for i=1,recap_temp.SelfListSize-1 do
				if recap_temp.SelfList[i][idx]~="0.0%" and recap_temp.SelfList[i][idx]~="--" and tostring(recap_temp.SelfList[i][idx])~="0" then
					if not hasdata and recap.Opt.SpamRows.value then
						DEFAULT_CHAT_FRAME:AddMessage(alltext)
					end
					hasdata = true
					text = string.sub(recap_temp.SelfList[i].EName,2)..(string.sub(recap_temp.SelfList[i].EName,1,1)=="3" and "*" or "")..(recap.Opt.SpamRows.value and ": " or " ")..recap_temp.SelfList[i][idx]..(idx=="ETotal" and (" ("..recap_temp.SelfList[i].ETotalP..")") or "")
					if not recap.Opt.SpamRows.value then
						alltext = alltext.." ["..text.."]"
					else
						DEFAULT_CHAT_FRAME:AddMessage(text)
					end
				end
			end
			if not recap.Opt.SpamRows.value and hasdata then
				DEFAULT_CHAT_FRAME:AddMessage(alltext)
			end
		end

	else
		DEFAULT_CHAT_FRAME:AddMessage(recap_temp.Local.RankUsage)
	end
end


function RecapSelf_OnClick(arg1)

	local id = this:GetID()
	local index = id + FauxScrollFrame_GetOffset(RecapScrollBar)
	local linkformat = recap_temp.Local.DamageDetailLink
	local text = ""
	local channel = ""

	if arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.SelfListSize and ChatFrameEditBox:IsVisible() then

		if string.sub(recap_temp.SelfList[index].EName,1,1)=="3" then
			linkformat = recap_temp.Local.HealDetailLink
		end
		text = string.format(linkformat,recap_temp.Player,string.sub(recap_temp.SelfList[index].EName,2),recap_temp.SelfList[index].ETotal,recap_temp.SelfList[index].ETotalP,recap_temp.Player )..", Max: "..recap_temp.SelfList[index].EMaxAll..((recap_temp.SelfList[index].ECritsP~="--") and (", Crit Rate: "..recap_temp.SelfList[index].ECritsP) or "")
		Recap_InsertChat(text," ")

	-- Hawksy: new post to "Self"
	elseif arg1=="LeftButton" and IsShiftKeyDown() and index<recap_temp.SelfListSize then

		if string.sub(recap_temp.SelfList[index].EName,1,1)=="3" then
			linkformat = recap_temp.Local.HealDetailLink
		end
		text = string.format(linkformat,recap_temp.Player,string.sub(recap_temp.SelfList[index].EName,2),recap_temp.SelfList[index].ETotal,recap_temp.SelfList[index].ETotalP,recap_temp.Player )..", Max: "..recap_temp.SelfList[index].EMaxAll..((recap_temp.SelfList[index].ECritsP~="--") and (", Crit Rate: "..recap_temp.SelfList[index].ECritsP) or "")
		DEFAULT_CHAT_FRAME:AddMessage(text," ")

	elseif arg1=="RightButton" and index<recap_temp.SelfListSize and IsControlKeyDown() and not IsShiftKeyDown() and ChatFrameEditBox:IsVisible() then
		-- Hawksy: post Recent Outgoing events for this effect for this combatant when Recent Data Mode is active
		if recap.Opt.RecentData.value then
			channel = ChatFrameEditBox:GetAttribute("chatType")
			Recap_PostRecentRows(true, recap_temp.Player, string.sub(recap_temp.SelfList[index].EName,2), channel)
		end

	elseif arg1=="RightButton" and index<recap_temp.SelfListSize and IsControlKeyDown() and not IsShiftKeyDown() then
		-- Hawksy: post Recent Outgoing events for this effect to Self channel if no chat edit box is visible
		if recap.Opt.RecentData.value then
			Recap_PostRecentRows(true, recap_temp.Player, string.sub(recap_temp.SelfList[index].EName,2), "Self")
		end

	elseif arg1=="RightButton" and index<recap_temp.SelfListSize and not IsShiftKeyDown() and not IsControlKeyDown() then
		this:LockHighlight()
		recap_temp.SelectedEffect = recap_temp.SelfList[index].EName
		RecapCreateMenu(recap_temp.EffectMenu, 1)
	end
end

-- Hawsky: new function to handle ParserLib events
function Recap_ParserLib_OnEvent(event, info)

if type(info) == "string" then
	-- Hawksy: saw some of these unexpected strings in early testing, but not recently, so perhaps it was a problem with an earlier version of ParserLib
	--   leaving in place for a while
	local debugMessage = "Recap_ParserLib_OnEvent info is unexpectedly a string, event: "..event.." info: "..info
	DumpMessage(debugMessage)
	DEFAULT_CHAT_FRAME:AddMessage("arg1 ("..tostring(arg1)..") -- "..debugMessage)
	if recap_temp.snap then
		Screenshot()
	end
else
		if event=="CHAT_MSG_COMBAT_FRIENDLY_DEATH" or event=="CHAT_MSG_COMBAT_HOSTILE_DEATH" then
			Recap_DeathEvent(event, info)
		elseif (info.type == "hit") or (info.type == "miss") then
			Recap_DamageEvent(event, info)
		elseif (info.type == "heal") then
			Recap_HealEvent(event, info)
		else
			-- Hawksy: currently ignoring anything else
		end
end
end

-- Hawksy: moved some events to Recap_ParserLib_OnEvent
function Recap_Combat_OnEvent()

	local thisCombatant, damageEffect, healEffect, event_time, combatantOkay

	if event=="UPDATE_MOUSEOVER_UNIT" then
		Recap_GetUnitInfo("mouseover")
	elseif event=="PLAYER_TARGET_CHANGED" then
		Recap_GetUnitInfo("target")

	-- Hawksy: extensively rewritten
	elseif (event=="UNIT_SPELLCAST_SUCCEEDED") then
		-- this event is said to occur for anyone in the party or raid, and perhaps also the combatant's target
		--   does not report the target for the spell (only UNIT_SPELLCAST_SENT does that, and it occurs also for failed casts)
		--   includes a lot of extra noise, such as pet attacks, dismounting, adding an Argent Dawn commission, etc.
		--   (2007-01-05: not sure it is catching everything, need to check into it
		--   currently not checking for whether pets get this event
		thisCombatant = (UnitName(arg1))
		if thisCombatant and recap.Combatant[thisCombatant] and (not recap.Combatant[thisCombatant].Ignore) then
			-- we have data for this combatant
			damageEffect = "1"..arg2
			healEffect = "3"..arg2
			if recap.Combatant[thisCombatant].Detail[damageEffect] or recap.Combatant[thisCombatant].Detail[healEffect] then
				-- this is a cast with which this combatant has done damage or healing before
				if recap.dump then
					DumpMessage("CAST;"..event..";"..thisCombatant..";"..string.gsub(arg2,"\ ","\_"))
				end

				event_time = GetTime()

				-- perhaps start the fight
				if not recap.Opt.LimitFights.value then
					if recap.Opt.IdleFight.value then
						recap_temp.IdleTimer = 0
					end
					if recap.Opt.State.value=="Idle" then
						Recap_StartFight()
					end
				end

				-- Hawksy: if in Self Data Mode check whether the combatant is the player (or players's pet)
				combatantOkay = Recap_CheckSelfMode(thisCombatant, (recap.Combatant[thisCombatant] and recap.Combatant[thisCombatant].OwnedBy))

				if combatantOkay then
					-- perhaps add the combatant into the fight, and update timers
					-- this takes the time from the moment of successful cast, rather than when it strikes the target
					if recap.Combatant[thisCombatant].Detail[damageEffect] then
						Recap_CreateCombatantAndUpdateTimers(thisCombatant, event_time, "Out")
						if recap.Combatant[thisCombatant].Detail[damageEffect].TicksDmg then
							-- this is the casting of a DoT, so count it as a hit
							--   (to register a 'hit' for a DoT, add to CritsEvents (!) (okay, I didn't design this scheme))
							recap.Combatant[thisCombatant].Detail[damageEffect].CritsEvents = (recap.Combatant[thisCombatant].Detail[damageEffect].CritsEvents or 0) + 1
							if thisCombatant==recap_temp.Player then
								recap.Self[recap_temp.s][damageEffect].CritsEvents = (recap.Self[recap_temp.s][damageEffect].CritsEvents or 0) + 1
							end

							-- Hawksy: support for Recent Data Mode
							if recap.Opt.RecentData.value then
								Recap_AddRecentEvent(event_time, thisCombatant, "?", 1, arg2, "cast", nil)
							end
						end
					end

					-- for heals, just update the timers
					-- a standard heal will get counted as it lands, and a HoT only counts ticks
					if recap.Combatant[thisCombatant].Detail[healEffect] then
						Recap_CreateCombatantAndUpdateTimers(thisCombatant, event_time, "Heal")
					end
				end
			end
		end
	end
end

function Recap_Register_CombatEvents()

	-- Hawksy: get pointer to ParserLib
	recap_temp.parserlib = ParserLib:GetInstance("1.1")

	if not recap_temp.CombatEventsRegistered then
	-- register damage events
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_PARTY_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_PARTY_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PARTY_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PARTY_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PET_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_PET_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_SELF_HITS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_SELF_MISSES", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_OTHERS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PET_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_SELF_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE", "Recap_ParserLib_OnEvent")

		-- Hawksy: added event
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FAILED_LOCALPLAYER", "Recap_ParserLib_OnEvent")

		-- register death events
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_COMBAT_HOSTILE_DEATH", "Recap_ParserLib_OnEvent")

		-- register healing events
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_PARTY_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PARTY_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PET_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_SELF_BUFF", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS", "Recap_ParserLib_OnEvent")
		recap_temp.parserlib:RegisterEvent("Recap", "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS", "Recap_ParserLib_OnEvent")

		RecapCombatEvents:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
		RecapCombatEvents:RegisterEvent("PLAYER_TARGET_CHANGED")
		-- Hawksy: change of event
		RecapCombatEvents:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")

		recap_temp.CombatEventsRegistered = 1
	end
end

function Recap_Unregister_CombatEvents()
	if recap_temp.CombatEventsRegistered then
		RecapCombatEvents:UnregisterAllEvents()
		recap_temp.parserlib:UnregisterAllEvents("Recap")
		recap_temp.CombatEventsRegistered = nil
	end
end